/*
    Copyright 2008 - 2011 Denis Comtesse <comtesse@users.sourceforge.net>

    This file is part of F.L.A.W.

    F.L.A.W. is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    F.L.A.W. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with F.L.A.W.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "fireball.h"


void Fireball::init (SDL_Surface *flysprites[10], SDL_Surface *startsprites[9], SDL_Surface *endsprites[8], SDL_Surface *glowsprites[10]){
	
	for (Sint8 f = 0; f < 10; f++){
		fly[f] = flysprites[f];
	}
	
	for (Sint8 f = 0; f < 10; f++){
		glow[f] = glowsprites[f];
	}	
	
	for (Sint8 f = 0; f < 16; f++){
		start[f] = startsprites[f];
	}
	
	for (Sint8 f = 0; f < 16; f++){
		end[f] = endsprites[f];
	}
	
	w = fly[0]->w;
	h = fly[0]->h;
	rect.w = w;
	rect.h = h;
	slowdown = false;	
	blitted = false;
}


void Fireball::statusupdate(void){

	if (status < 300){	
		if (status >= 200) {	//END	
			frame = (status - 200)/2;		
			status++;
			if (status > 231){      
				status = 300;
			}
		}

		else if (status >= 150){				//GLOW
			frame = (status - 150)/4;		
			status++;
			if (status > 189){      
				status = 100;
			}		
		}

		else if (status >= 100){				//FLY	
			frame = (status - 100)/2;		
			status++;
			if (status > 119){      
				status = 100;
			}
		}

		else if (status < 100) {				//START
			frame = status/2;		
			status++;
			if (status > 31){
				status = 100;
			}
		}

		if (status > 99) {
			x+=speed.x;
			y+=speed.y;
		}
		
		if (slowdown==true) {
			speed.x*=0.996;
			speed.y*=0.996;
		}
	}
}

void Fireball::bordercollision (void) {
	if (x > 600) {
		if (speed.x > 0) speed.x *= -1;
		if (status < 300) collision = true;
	}
	
	else if (x < 48) {
		if (speed.x < 0) speed.x *= -1;
		if (status < 300) collision = true;
	}
	
	if (y > 436) {
		if (speed.y > 0) speed.y *= -1;
		if (status < 300) collision = true;
	}
	
	else if (y < 26) {
		if (speed.y < 0) speed.y *= -1;
		if (status < 300) collision = true;
	}
}



//blit:

void Fireball::blit (float screenfactor, float screenoffset) {

	SDL_Surface *dst = SDL_GetVideoSurface();
	rect.x = screenfactor*x-w/2+screenoffset;
	rect.y = screenfactor*y-h/2;
	
	if (status < 300){	
		if (status >= 200){
			SDL_BlitSurface (end[frame], NULL, dst, &rect);
			blitted = true;
		}	
		
		else if (status >= 150){
			SDL_BlitSurface (glow[frame], NULL, dst, &rect);
			blitted = true;
		}	
		
		else if (status >= 100){
			SDL_BlitSurface (fly[frame], NULL, dst, &rect);
			blitted = true;
		}
		
		else {
			SDL_BlitSurface (start[frame], NULL, dst, &rect);
			blitted = true;
		}
	}
}


//restore background:

void Fireball::blitrestore(SDL_Surface *backup, float screenoffset){
	SDL_Surface *dst = SDL_GetVideoSurface();
	if (blitted == true) {
		SDL_Rect srcrect = rect;
		srcrect.x -= screenoffset;
		SDL_BlitSurface (backup, &srcrect, dst, &rect);
		blitted = false;
	}
}



