/*
    Copyright 2008 - 2011 Denis Comtesse <comtesse@users.sourceforge.net>

    This file is part of F.L.A.W.

    F.L.A.W. is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    F.L.A.W. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with F.L.A.W.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "game.h"

#ifdef __linux__
//#	define PATH "data/"
#	define PATH PACKAGE_DATA_DIR"/flaw/"
#else
#	define PATH "data/"
#endif


// ******* setup functions *******

void Game::DisplaySetup(void){

	hq = true; //removed useless "quality" option
	
	if (res == 0) {
		screenfactor = 0.5;
		screenoffset = 0;
		xres = 320;
		yres = 240;
	}
	else if (res == 1) {
		screenfactor = 0.825;
		screenoffset = 56;
		xres = 640;
		yres = 400;
	}
	else if (res == 2) {
		screenfactor = 1;
		screenoffset = 0;
		xres = 640;
		yres = 480;
	}
	else if (res == 3) {
		screenfactor = 1.25;
		screenoffset = 0;
		xres = 800;
		yres = 600;
	}
	else if (res == 4) {
		screenfactor = 1.5;
		screenoffset = 160;
		xres = 1280;
		yres = 720;
	}
	else if (res == 5) {
		screenfactor = 1.6;
		screenoffset = 0;
		xres = 1024;
		yres = 768;
	}
	else if (res == 6) {
		screenfactor = 1.6;
		screenoffset = 171;
		xres = 1366;
		yres = 768;
	}
	else if (res == 7) {
		screenfactor = 1.65;
		screenoffset = 112;
		xres = 1280;
		yres = 800;
	}
	else if (res == 8) {
		screenfactor = 2;
		screenoffset = 0;
		xres = 1280;
		yres = 960;
	}
	else if (res == 9) {
		screenfactor = 2;
		screenoffset = 0;
		xres = 1280;
		yres = 1024;
	}
	else {
		screenfactor = 0.5;
		screenoffset = 0;
		xres = 640;
		yres = 480;
	}

	if (fullscreen) {
		screen = SDL_SetVideoMode(xres, yres, 32, SDL_SWSURFACE |  SDL_FULLSCREEN );
		if (screen == NULL) {
			screen = SDL_SetVideoMode(xres, yres, 24, SDL_SWSURFACE |  SDL_FULLSCREEN );	
			if (screen == NULL) {
				screen = SDL_SetVideoMode(xres, yres, 16, SDL_SWSURFACE |  SDL_FULLSCREEN );			
				if (screen == NULL) {
					screen = SDL_SetVideoMode(xres, yres, 15, SDL_SWSURFACE |  SDL_FULLSCREEN );			
					if (screen == NULL) {
						cout << "Can't set video mode: %s\n" << SDL_GetError();
						exit(1);
					}
				}
			}
		}
	}
	
	else{
		screen = SDL_SetVideoMode(xres, yres, 32, SDL_SWSURFACE);
		if (screen == NULL) {
			screen = SDL_SetVideoMode(xres, yres, 16, SDL_SWSURFACE);			
			if (screen == NULL) {
				screen = SDL_SetVideoMode(xres, yres, 15, SDL_SWSURFACE);			
				if (screen == NULL) {
					cout << "Can't set video mode: %s\n" << SDL_GetError();
					exit(1);
				}
			}
		}
	}


	SDL_WM_SetCaption("F.L.A.W.", "F.L.A.W.");
	SDL_ShowCursor(SDL_ENABLE);

}


void Game::AudioSetup(){
	Mix_CloseAudio();

	if (Mix_OpenAudio(22050, MIX_DEFAULT_FORMAT, 2, 2048) < 0){
		cerr << "Error initializing SDL_mixer: %s\n", Mix_GetError();
		exit(1);
	}

	Mix_AllocateChannels(8);
	swing = Mix_LoadWAV(PATH"audio/swing.ogg");
	hit = Mix_LoadWAV(PATH"audio/hit.ogg");
	collide = Mix_LoadWAV(PATH"audio/fireball.ogg");
	die = Mix_LoadWAV(PATH"audio/die.ogg");
	shoot = Mix_LoadWAV(PATH"audio/shoot.ogg");
	invoke = Mix_LoadWAV(PATH"audio/invoke.ogg");
	newgem = Mix_LoadWAV(PATH"audio/gem.ogg");
	wah = Mix_LoadWAV(PATH"audio/wah.ogg");
	
	extrasound[1] = Mix_LoadWAV(PATH"audio/extra1.ogg");
	extrasound[2] = Mix_LoadWAV(PATH"audio/extra2.ogg");
	extrasound[3] = Mix_LoadWAV(PATH"audio/extra3.ogg");
	extrasound[4] = Mix_LoadWAV(PATH"audio/extra4.ogg");

	gamemusic=Mix_LoadMUS(PATH"audio/gamemusic.xm");
	titlemusic=Mix_LoadMUS(PATH"audio/titlemusic.xm");
	Mix_Volume(7, 16);
	soundchannel = 0;

	for (Uint8 n = 0; n < 7; n++){
		Mix_Volume(n, 32);
	}


}

void Game::StartMusic(void){
	if (playmusic){
		Uint8 n = randomnumber(1,3);
		Mix_VolumeMusic(128);
		Mix_PlayMusic(gamemusic, -1);
		if (n == 2){
			Mix_SetMusicPosition(0x14);
		}
		else if (n == 3){
			Mix_SetMusicPosition(0x29);
		}
		
	}
}

void Game::LoadArena(void){

	SDL_Surface *scalecopy;
	if (arenatype == 1){
		if (!(scalecopy = IMG_Load (PATH"arenas/arena-stone.png"))){
			cout << IMG_GetError();
			exit(1);
		}
	}
	else if (arenatype == 2){
		if (!(scalecopy = IMG_Load (PATH"arenas/arena-ice.png"))){
			cout << IMG_GetError();
			exit(1);
		}
	}
	else if (arenatype == 3) {
		if (!(scalecopy = IMG_Load (PATH"arenas/arena-fire.png"))){
			cout << IMG_GetError();
			exit(1);
		}
	}
	else {
		if (!(scalecopy = IMG_Load (PATH"arenas/arena-ruins.png"))){
			cout << IMG_GetError();
			exit(1);
		}
	}
	
	if (screenfactor == 2) arena = scalecopy;
	else {
		arena = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, hq);
		SDL_FreeSurface(scalecopy);
	}
	    
	SDL_SetAlpha(arena, SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
}

void Game::LoadGfx(void){

	SDL_Surface *scalecopy;
	
	//Load Gem:
	if (!(scalecopy = IMG_Load (PATH"sprites/extras/gem.png"))){
		cout << IMG_GetError();
		exit(1);
	}
	if (screenfactor == 2) gem = scalecopy;
	else {
		gem = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, hq);
		SDL_FreeSurface(scalecopy);
	}
	if (hq == true) SDL_SetAlpha(gem, SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
	else gem=killalpha(gem, 255);

	//Load Fireballs:
	for (Sint8 f = 0; f < 10; f++){
		char filename[60];
		sprintf (filename, PATH"fireball/fly_%02d.png", f+1);

		if (!(scalecopy = IMG_Load (filename))){
			cout << IMG_GetError();
			exit(1);
		}

		if (screenfactor == 2) fireballfly[f] = scalecopy;
		else {
			fireballfly[f] = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, hq);
			SDL_FreeSurface(scalecopy);
		}
		if (hq == true) SDL_SetAlpha(fireballfly[f], SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
		else fireballfly[f] = killalpha(fireballfly[f],255);
	}

	for (Sint8 f = 0; f < 10; f++){
		char filename[60];
		sprintf (filename, PATH"fireball/glow_%02d.png", f+1);

		if (!(scalecopy = IMG_Load (filename))){
			cout << IMG_GetError();
			exit(1);
		}

		if (screenfactor == 2) fireballglow[f] = scalecopy;
		else {
			fireballglow[f] = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, hq);
			SDL_FreeSurface(scalecopy);
		}
		if (hq == true) SDL_SetAlpha(fireballglow[f], SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
		else fireballglow[f] = killalpha(fireballglow[f],255);
	}

	for (Sint8 f = 0; f < 16; f++){
		char filename[60];
		sprintf (filename, PATH"fireball/start_%02d.png", f+1);

		if (!(scalecopy = IMG_Load (filename))){
			cout << IMG_GetError();
			exit(1);
		}

		if (screenfactor == 2) fireballstart[f] = scalecopy;
		else {
			fireballstart[f] = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, hq);
			SDL_FreeSurface(scalecopy);
		}
		if (hq == true) SDL_SetAlpha(fireballstart[f], SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
		else fireballstart[f] = killalpha(fireballstart[f],255);
	}

	for (Sint8 f = 0; f < 16; f++){
		char filename[60];
		sprintf (filename, PATH"fireball/end_%02d.png", f+1);

		if (!(scalecopy = IMG_Load (filename))){
			cout << IMG_GetError();
			exit(1);
		}

		if (screenfactor == 2) fireballend[f] = scalecopy;
		else {
			fireballend[f] = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, hq);
			SDL_FreeSurface(scalecopy);
		}
		if (hq == true) SDL_SetAlpha(fireballend[f], SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
		else fireballend[f] = killalpha(fireballend[f],128);
	}

	for (Sint8 n = 0; n < 10; n++){
		fireball[n].init(fireballfly, fireballstart, fireballend, fireballglow);
	}

	//Load Wizards
	wizardgfx[0].load("blue", false, screenfactor, hq);
	wizardgfx[1].load("red", false, screenfactor, hq);
	wizardgfx[2].load("green", false, screenfactor, hq);
	wizardgfx[3].load("black", false, screenfactor, hq);
	wizardgfx[4].load("purple", false, screenfactor, hq);
	wizardgfx[5].load("orange", false, screenfactor, hq);
	wizardgfx[6].load("ghost", true, screenfactor, hq);

	for (Uint8 n = 0; n < 5;n++){
		wizard[n].setup(n, screenfactor);
	}
}


void Game::RandomizeAI(void){
	for (Uint8 n = 0; n < numplayers; n++){
		wizard[n].aggressive = double(randomnumber(5,10))/10;
		wizard[n].idle = double(randomnumber(5,10))/10;
		wizard[n].risk = double(randomnumber(5,10))/10;
		wizard[n].fear = double(randomnumber(5,10))/10;
		wizard[n].reaction = randomnumber(4,6)-difficulty;
	}
}


void Game::SetStartPositions(void){
	numfireballs = 0;
	wizard[0].x = 248;
	wizard[0].y = 144;
	wizard[0].direction = DOWNRIGHT;
	wizard[1].x = 440;
	wizard[1].y = 270;
	wizard[1].direction = LEFT;
	wizard[2].x = 200;
	wizard[2].y = 272;
	wizard[2].direction = RIGHT;
	wizard[3].x = 392;
	wizard[3].y = 144;
	wizard[3].direction = DOWNLEFT;
	wizard[4].x = 320;
	wizard[4].y = 368;
	wizard[4].direction = UP;
	
	for (Uint8 n=0; n < 5; n++){
		wizardsorted[n] = &wizard[n];
	}

	for (Uint8 n = 0; n < numplayers; n++){
		wizard[n].status = 0;
		wizard[n].control.x = 0;
		wizard[n].control.y = 0;
		wizard[n].control.fire = 0;
		wizard[n].aidelay = randomnumber(0,3);
		wizard[n].aiprior.x = 0;
		wizard[n].aiprior.y = 0;
		wizard[n].extra = 0;
		wizard[n].timer = 0;
		wizard[n].follows = -1;
		wizard[n].prevspeed.x = 0;
		wizard[n].prevspeed.y = 0;
		if (arenatype == 2) wizard[n].skid = true;
		else wizard[n].skid = false;
	}

	
	for (Uint8 n = 0; n < 10; n++){
		fireball[n].x = 320;
		fireball[n].y = 240;
		fireball[n].status = 0;
		fireball[n].frame = 0;
		fireball[n].speed.x = randomnumber(20,36)/8;
		fireball[n].speed.y = randomnumber(20,36)/8;
		if (tactical) fireball[n].slowdown = true;
		else fireball[n].slowdown = false;
		bool neg = (rand()%2);
		if (neg == true) fireball[n].speed.x *= -1;
		neg = (rand()%2);
		if (neg == true) fireball[n].speed.y *= -1;
	}
	
	if (arenatype == 3){
		numfireballs = 3;
		fireball[0].x = 320;
		fireball[0].y = 80;
		fireball[1].x = 160;
		fireball[1].y = 360;
		fireball[2].x = 480;
		fireball[2].y = 360;
		for (Uint8 n = 0; n < 3; n++){
			fireball[n].speed.x = 0;
			fireball[n].speed.y = 0;
			fireball[n].status = 100;
		}
	}

	if (arenatype == 4){
		hole[0].x = 316;
		hole[0].y = 106;
		hole[1].x = 514;
		hole[1].y = 128;
		hole[2].x = 479;
		hole[2].y = 345;	
		hole[3].x = 115;
		hole[3].y = 387;	
		hole[4].x = 92;
		hole[4].y = 78;
		for (Uint8 n = 0; n<5; n++){
			hole[n].radius = 28;
		}
	}
			
		
	extra.on = false;
	extra.blitted = false;
	extra.newgem = false;
	extra.x = 0;
	extra.y = 0;
	extra.rect.w = 15*screenfactor;
	extra.rect.h = 15*screenfactor;
}


// ******* mainloop functions *******

void Game::Events(void){

	Uint8 *keystate;
	SDL_PollEvent(&event);
	keystate = SDL_GetKeyState (0);

	if (event.type == SDL_QUIT){
		SDL_Quit();
		exit(0);
	}

	if (keystate[SDLK_ESCAPE]){ 	//quit when ESC is pressed
		stop = true;
		quit = true;
	}
	else if (keystate[SDLK_p]) Pause();

	for (Uint8 n = 0; n < numplayers; n++) {
		wizard[n].updatecontroller();
	}
}



void Game::DisplayUpdate(void) {

	if (!stop) SortWizards();


	for (Sint8 n = numfireballs-1; n >= 0; n--) {
		fireball[n].blitrestore(arena, screenoffset);
	}

	for (Sint8 n = numplayers-1; n >= 0; n--) {
		wizardsorted[n]->blitrestore(arena, screenoffset);
	}

	if (extra.blitted) {
		SDL_Rect arenarect;
		arenarect.x = screenfactor*extra.x-extra.rect.w/2;
		arenarect.y = screenfactor*extra.y-extra.rect.h/2;
		arenarect.w = extra.rect.w;
		arenarect.h = extra.rect.h;
		SDL_BlitSurface (arena, &arenarect, screen, &extra.rect);
		extra.blitted = false;
	}

	if (extra.on) {
		extra.rect.x = screenfactor*extra.x-extra.rect.w/2+screenoffset;
		extra.rect.y = screenfactor*extra.y-extra.rect.h/2;
		SDL_BlitSurface(gem, NULL, screen, &extra.rect);
		extra.blitted = true;
	}

	for (Sint8 n = 0; n < numplayers; n++) {

		if (wizardsorted[n]->extra == 1) {
			if ((wizardsorted[n]->timer < 50)&&(wizardsorted[n]->timer%2 == 0)) wizardsorted[n]->blit(&wizardgfx[wizardsorted[n]->gfxset], screenfactor, screenoffset);
			else wizardsorted[n]->blit(&wizardgfx[5], screenfactor, screenoffset);
		}
		else if (wizardsorted[n]->extra == 4) {
			if ((wizardsorted[n]->timer < 50)&&(wizardsorted[n]->timer%2 == 0)) wizardsorted[n]->blit(&wizardgfx[wizardsorted[n]->gfxset], screenfactor, screenoffset);
			else wizardsorted[n]->blit(&wizardgfx[6], screenfactor, screenoffset);
		}
		else wizardsorted[n]->blit(&wizardgfx[wizardsorted[n]->gfxset], screenfactor, screenoffset);
	}

	for (Sint8 n = 0; n < numfireballs; n++) {
		if ((!stop)&&(fireball[n].status < 300)) fireball[n].blit(screenfactor, screenoffset);
	}

	SDL_Flip (screen);
}


//check if the match is over
void Game::CheckState(void){
	if (numdead >= numplayers-1){

		if (numfireballs == 0) {
			stop = true;
		}
		else if (fireball[0].status < 200){
			for (Uint8 n = 0; n < numfireballs; n++){
				fireball[n].status = 200;
			}
		}
		else if (fireball[0].status == 210){
			for (Uint8 n = 0; n < numplayers; n++){
				if (wizard[n].timer > 5) wizard[n].timer == 5;
			}
		}
		else if (fireball[0].status == 300){
			stop = true;
		}
	}
}

void Game::GameUpdate(){

	
	CheckState ();
	
	//launch new fireballs
	if ((numfireballs < 10) && (numdead < numplayers-1)){
		fireballtimer++;
		if (fireballtimer > framerate.framespersecond*10){
			fireballtimer = 0;
			numfireballs++;
		}
	}

	//place gem
	if ((randomnumber(0, 1000) == 0) && !(extra.on) && (gems)) {
		bool valid;
		extra.on = true;
		extra.newgem = true;

		do {
			valid = true;
			extra.x = rand()%540 + 50;
			extra.y = rand()%390 + 40;

			for (Uint8 n = 0; n < numplayers; n++){
				float dist = sqrt((extra.x-wizard[n].x)*(extra.x-wizard[n].x)+(extra.y-wizard[n].y)*(extra.y-wizard[n].y));
				if (dist < 50) valid = false;
			}
			if (arenatype == 4){
				for (Uint8 m=0; m < 5; m++){
					float dist = sqrt((hole[m].x-extra.x)*(hole[m].x-extra.x)+(hole[m].y-extra.y)*(hole[m].y-extra.y));
					if (dist < hole[m].radius+10) valid = false;
				}
			}

		}while (!valid);
	}




	for (Uint8 n = 0; n < numfireballs; n++) {

		//two fireballs collide
		for (Sint8 m = n+1; m < numfireballs; m++){

			float dist = distance(&fireball[n], &fireball[m]);

			if ((dist < 28) && (fireball[m].status >= 100)) {
				fireball[n].collision = true;
				int zx = fireball[n].speed.x;
				int zy = fireball[n].speed.y;
				/*
				fireball[n].speed.x = 0.05*zx + fireball[m].speed.x;
				fireball[n].speed.y = 0.05*zy + fireball[m].speed.y;
				fireball[m].speed.x = zx + 0.05*fireball[m].speed.x;
				fireball[m].speed.y = zy + 0.05*fireball[m].speed.y;
				*/				
				fireball[n].speed.x = fireball[m].speed.x;
				fireball[n].speed.y = fireball[m].speed.y;
				fireball[m].speed.x = zx;
				fireball[m].speed.y = zy;
				
				if (abs(fireball[n].speed.x) < 0.3) fireball[n].speed.x += 0.05*fireball[m].speed.x;
				if (abs(fireball[n].speed.y) < 0.3) fireball[n].speed.y += 0.05*fireball[m].speed.y;
				if (abs(fireball[m].speed.x) < 0.3) fireball[m].speed.x += 0.05*fireball[n].speed.x;
				if (abs(fireball[m].speed.y) < 0.3) fireball[m].speed.y += 0.05*fireball[n].speed.y;
				
				if (fireball[m].x > fireball[n].x){
					fireball[m].x++;
					fireball[n].x--;
				}
				else {
					fireball[m].x--;
					fireball[n].x++;
				}
				if (fireball[m].y > fireball[n].y){
					fireball[m].y++;
					fireball[n].y--;
				}
				else {
					fireball[m].y--;
					fireball[n].y++;
				}
			}
		}

		//fireball collides with wizard

		for (Sint8 m = 0; m < numplayers; m++){
			float dist = distance(&wizard[m], &fireball[n]);
			//shield reflects fireballs:
			if (wizard[m].extra == 2) {
				if ((wizard[m].status < 400) && (fireball[n].status >= 100)){
					if (dist < 15) {
						fireball[n].collision = true;
						if ((Sgn(wizard[m].x - fireball[n].x)) == Sgn(fireball[n].speed.x)) {
							fireball[n].speed.x *= -1;
						}
						if ((Sgn(wizard[m].y - fireball[n].y)) == Sgn(fireball[n].speed.y)) {
							fireball[n].speed.y *= -1;
						}
						if (abs(fireball[n].speed.x) < abs(wizard[m].speed.x)){
								fireball[n].speed.x = wizard[m].speed.x*1.5;
						}
						if (abs(fireball[n].speed.y) < abs(wizard[m].speed.y)){
								fireball[n].speed.y = wizard[m].speed.y*1.5;
						}
					}
					else if (dist < 35) {
						fireball[n].collision = true;
						if (((Sgn(wizard[m].x - fireball[n].x)) == Sgn(fireball[n].speed.x)) && (abs(wizard[m].x - fireball[n].x) > 10)) {
							fireball[n].speed.x *= -1;
						}
						if (((Sgn(wizard[m].y - fireball[n].y)) == Sgn(fireball[n].speed.y)) && (abs(wizard[m].y - fireball[n].y) > 10)) {
							fireball[n].speed.y *= -1;
						}
						if (abs(fireball[n].speed.x) < abs(wizard[m].speed.x)){
								fireball[n].speed.x = wizard[m].speed.x*1.5;
						}
						if (abs(fireball[n].speed.y) < abs(wizard[m].speed.y)){
								fireball[n].speed.y = wizard[m].speed.y*1.5;
						}
					}
				}
			}

			else {
				if ((dist < 22) && (wizard[m].status < 400) && (fireball[n].status >= 100) && (fireball[n].status < 208)){
					wizard[m].status = 400;
					numdead++;
				}
			}
		}

		fireball[n].bordercollision();
	}

	for (Uint8 n = 0; n < numplayers; n++) {

		//Wizard collects gem
		float dist = sqrt((extra.x-wizard[n].x)*(extra.x-wizard[n].x)+(extra.y-wizard[n].y)*(extra.y-wizard[n].y));
		if ((dist < 13) && (extra.on)){
			extra.on = false;
			Uint8 random = randomnumber(0, 99);

			if (random < 20) {		//death wizard
				wizard[n].extra = 1;
				wizard[n].timer = 500;
				bool done = false;
				do{
					Uint8 m = randomnumber(0, numplayers);
					if ((m!=n) && (wizard[m].status < 400)){
						wizard[n].follows = m;
						done = true;
					}
				}while ((!done)&&(numdead < numplayers-1));
			}

			else if (random < 40) {	//shield
				wizard[n].extra = 2;
				wizard[n].timer = 400;
			}

			else if (random < 70) {	//speed
				wizard[n].extra = 3;
				wizard[n].timer = 600;
			}

			else {					//ghost wizard
				wizard[n].extra = 4;
				wizard[n].timer = 500;
				wizard[n].prevspeed.x = 0;
				wizard[n].prevspeed.y = 0;
			}

		}

		//Wizard falls into hole
		if ((wizard[n].status < 400) && (wizard[n].extra != 4) && (arenatype == 4)) {
			for (Uint8 m=0; m < 5; m++){
				float dist = sqrt((hole[m].x-wizard[n].x)*(hole[m].x-wizard[n].x)+(hole[m].y-wizard[n].y)*(hole[m].y-wizard[n].y));
				if (dist < hole[m].radius){
					wizard[n].status = 450;
					numdead++;
				}
				else if ((dist < hole[m].radius+10)&&(wizard[n].status > 320)){
					wizard[n].status = 450;
					numdead++;
					wizard[n].direction = (wizard[n].direction+4)%8;
				}
			}
		}
		
		//Gradually move falling wizard into the center
		if ((wizard[n].status < 500) && (wizard[n].status >= 450)){
			for (Uint8 m=0; m < 5; m++){
				float dist = sqrt((hole[m].x-wizard[n].x)*(hole[m].x-wizard[n].x)+(hole[m].y-wizard[n].y)*(hole[m].y-wizard[n].y));
				if (dist < hole[m].radius+10){
					wizard[n].x = (wizard[n].x*4+hole[m].x)/5;
					wizard[n].y = (wizard[n].y*4+hole[m].y+8)/5;		
				}
			}	
	    }
		
		//Wizard hits Wizard
		if ((wizard[n].status == 206) && (wizard[n].extra != 2)){
			for (Uint8 m = 0; m < numplayers; m++){
				if ((n != m) && (faces(&wizard[n], &wizard[m])) && (wizard[m].extra != 4) && ((wizard[m].extra != 2) || (wizard[n].extra = 4))){
					if ((wizard[m].status < 300)&&(wizard[n].extra != 1)) {
						wizard[m].status = 300;
						wizard[m].direction = (wizard[n].direction + 4) % 8;

						//fall correction (don't fall "into" the walls)
						if (wizard[m].x > 590) {
							if (wizard[m].direction == 5) wizard[m].direction = 4;
							if (wizard[m].direction == 6) wizard[m].direction = 2;
							if (wizard[m].direction == 7) wizard[m].direction = 0;
						}
						if (wizard[m].x < 58) {
							if (wizard[m].direction == 3) wizard[m].direction = 4;
							if (wizard[m].direction == 2) wizard[m].direction = 6;
							if (wizard[m].direction == 1) wizard[m].direction = 0;
						}
						if (wizard[m].y > 422) {
							if (wizard[m].direction == 1) wizard[m].direction = 2;
							if (wizard[m].direction == 0) wizard[m].direction = 4;
							if (wizard[m].direction == 7) wizard[m].direction = 6;
						}
						if (wizard[m].y < 32) {
							if (wizard[m].direction == 3) wizard[m].direction = 2;
							if (wizard[m].direction == 4) wizard[m].direction = 0;
							if (wizard[m].direction == 5) wizard[m].direction = 6;
						}
					}
					if ((wizard[m].status < 400)&&(wizard[n].extra == 1)) {   //hit by fire wizard
						wizard[m].status = 400;
						numdead++;
					}
				}
			}
		}

		//Wizard hits Fireball
		if ((wizard[n].status > 204) && (wizard[n].status < 208) && (wizard[n].extra != 2)){
			for (int m = 0; m < numfireballs; m++){
				if ((faces(&wizard[n], &fireball[m]))&&(fireball[m].status<200)){
					if ((fireball[m].status < 150) || (fireball[m].status > 153)){
						fireball[m].speed.x /= 2;
						fireball[m].speed.y /= 2;
						fireball[m].status = 150;

						switch (wizard[n].direction) {

							case UP:
								fireball[m].speed.y -= 7;
								break;

							case UPRIGHT:
								fireball[m].speed.x += 5;
								fireball[m].speed.y -= 5;
								break;

							case RIGHT:
								fireball[m].speed.x += 7;
								break;

							case DOWNRIGHT:
								fireball[m].speed.x += 5;
								fireball[m].speed.y += 5;
								break;

							case DOWN:
								fireball[m].speed.y += 7;
								break;

							case DOWNLEFT:
								fireball[m].speed.x -= 5;
								fireball[m].speed.y += 5;
								break;

							case LEFT:
								fireball[m].speed.x -= 7;
								break;

							case UPLEFT:
								fireball[m].speed.x -= 5;
								fireball[m].speed.y -= 5;
								break;
						}
					}
				}
			}
		}

		//Wizards collide:
		for (int m = 0; m < numplayers; m++){
			if ((n != m) && (collides(&wizard[n], &wizard[m]))){

				if (wizard[m].status < 400){
					if ((wizard[n].x > wizard [m].x) && ((wizard[n].control.x < 0)||(wizard[n].prevspeed.x < 0))){
						wizard[n].control.x = 0;
						wizard[n].prevspeed.x = 0;
					}
					else if ((wizard[n].x < wizard [m].x) && ((wizard[n].control.x > 0)||(wizard[n].prevspeed.x > 0))){
						wizard[n].control.x = 0;
						wizard[n].prevspeed.x = 0;
					}

					if ((wizard[n].y > wizard [m].y) && ((wizard[n].control.y < 0)||(wizard[n].prevspeed.y < 0))){
						wizard[n].control.y = 0;
						wizard[n].prevspeed.y = 0;
					}
					else if ((wizard[n].y < wizard [m].y) && ((wizard[n].control.y > 0)||(wizard[n].prevspeed.y > 0))){
						wizard[n].control.y = 0;
						wizard[n].prevspeed.y = 0;
					}
				}
			}
		}

		// calculate idletime
		if ((wizard[n].speed.x == 0) && (wizard[n].speed.y == 0) && (wizard[n].control.fire == false)) {
			wizard[n].idletime++;
		}
		else {
			wizard[n].idletime = 0;
		}
	}
	for (Uint8 n = 0; n < numplayers; n++) { //Update Wizards
		wizard[n].wallcollisions();
		wizard[n].statusupdate();
		wizard[n].setspeeds();
	}

	for (Uint8 m = 0; m < numfireballs; m++) { //Update Fireballs
		fireball[m].statusupdate();	
	}

	//make ghosts float (using sine wave)
	angle +=18;
	if (angle > 359) angle=0;
	for (Uint8 n = 0; n < numplayers; n++) {
		if (wizard[n].extra==4) wizard[n].yoffset = (sine[angle]*screenfactor*3);
	}

	
}


void Game::PlaySounds(){

	for (Sint8 n = numplayers-1; n >= 0; n--) {

		if (wizard[n].status == 201){
			Mix_PlayChannel(soundchannel, swing, 0);
			soundchannel++;
			if (soundchannel > 6) soundchannel = 0;
		}
		else if (wizard[n].status == 301){
			Mix_PlayChannel(soundchannel, hit, 0);
			soundchannel++;
			if (soundchannel > 6) soundchannel = 0;
		}

		else if (wizard[n].status == 401){
			Mix_PlayChannel(soundchannel, die, 0);
			soundchannel++;
			if (soundchannel > 6) soundchannel = 0;
		}
		
		else if (wizard[n].status == 451){
			Mix_PlayChannel(soundchannel, wah, 0);
			soundchannel++;
			if (soundchannel > 6) soundchannel = 0;
		}
		
		if ((wizard[n].extra == 1) && (wizard[n].timer == 499)) {
			Mix_PlayChannel(soundchannel, extrasound[1], 0);
			soundchannel++;
			if (soundchannel > 6) soundchannel = 0;
		}

		else if ((wizard[n].extra == 2) && (wizard[n].timer == 399)) {
			Mix_PlayChannel(soundchannel, extrasound[2], 0);
			soundchannel++;
			if (soundchannel > 6) soundchannel = 0;
		}

		else if ((wizard[n].extra == 3) && (wizard[n].timer == 599)) {
			Mix_PlayChannel(soundchannel, extrasound[3], 0);
			soundchannel++;
			if (soundchannel > 6) soundchannel = 0;
		}

		else if ((wizard[n].extra == 4) && (wizard[n].timer == 499)) {
			Mix_PlayChannel(soundchannel, extrasound[4], 0);
			soundchannel++;
			if (soundchannel > 6) soundchannel = 0;
		}
	}

	Uint8 collision = 0;

	for (Uint8 n = 0; n < numfireballs; n++) {
		if (fireball[n].collision == true){
			collision++;
			fireball[n].collision = false;
		}
		if (fireball[n].status == 151){
			Mix_PlayChannel(soundchannel, shoot, 0);
			soundchannel++;
			if (soundchannel > 6) soundchannel = 0;
		}
		if (fireball[n].status == 1){
			Mix_PlayChannel(7, invoke, 0);
		}
	}

	if (collision > 0){
		Mix_PlayChannel(soundchannel, collide, 0);
		soundchannel++;
		if (soundchannel > 6) soundchannel = 0;
	}

	if (extra.newgem){
		Mix_PlayChannel(soundchannel, newgem, 0);
		soundchannel++;
		if (soundchannel > 6) soundchannel = 0;
		extra.newgem = false;
	}

}


void Game::AI(){

	for (Uint8 n = 0; n < numplayers; n++) {
		wizard[n].aidelay = (wizard[n].aidelay +1)%wizard[n].reaction;

		if ((wizard[n].ai == true) && (wizard[n].aidelay == 0)){

			if ((wizard[n].follows >-1)&&(wizard[wizard[n].follows].status > 400)) {
				bool done = false;
				do{
					Uint8 m = randomnumber(0, numplayers);
					if ((m!=n) && (wizard[m].status < 400)){
						wizard[n].follows = m;
						done = true;
					}
				}while ((!done)&&(numdead < numplayers-1));
			}


			wizard[n].control.x = 0;
			wizard[n].control.y = 0;
			wizard[n].control.fire = false;

			float x = 0; //Movement priorities will be added
			float y = 0;
			float fire = 0;

			// Try to avoid the corners:
			if (wizard[n].extra != 1){
				float f;
				if (arenatype == 4) f = 0.2;
				else f = 1;
				if (wizard[n].x > 480) x -= pow((wizard[n].x - 480)/160, 2)*(1.5-wizard[n].fear)*750*f;
				if (wizard[n].x < 160) x += pow((160 - wizard[n].x)/160, 2)*(1.5-wizard[n].fear)*750*f;
				if (wizard[n].y > 320) y -= pow((wizard[n].y - 320)/160, 2)*(1.5-wizard[n].fear)*750*f;
				if (wizard[n].y < 160) y += pow((160 - wizard[n].y)/160, 2)*(1.5-wizard[n].fear)*750*f;
			}

			// Try to avoid the center:
			float centerdist = sqrt(pow((wizard[n].x-320),2)+pow((wizard[n].y-240),2));
			if (centerdist < 80){
				x += Sgn(wizard[n].x - 320) * (80 - centerdist)/80*(1.5-wizard[n].risk)*300;
				y += Sgn(wizard[n].y - 240) * (80 - centerdist)/80*(1.5-wizard[n].risk)*300;
			}

			// Move away from the fireballs:
			for (Uint8 m = 0; m < numfireballs; m++) {
				float dist = distance(&wizard[n], &fireball[m]);
				float xdist = abs(wizard[n].x-fireball[m].x);
				float ydist = abs(wizard[n].y-fireball[m].y);
				Uint8 forethought = 60 + difficulty*20;
				if ((abs(fireball[m].speed.x)+abs(fireball[m].speed.y) > 2)||(dist < 34)) {
					if ((dist < 320) && (wizard[n].extra!=2)) {

						if ((xdist < forethought)&&(fireball[m].speed.x != 0)) {
							if ((ydist > xdist) || (xdist < ydist)) { //consider fireball x movement:
								x -= Sgn(fireball[m].speed.x) * pow((320 - dist)/320, 3)/numfireballs*(wizard[n].fear)*2000;
							}
							else { //consider fireball x position:
								x += Sgn(wizard[n].x - fireball[m].x) * pow((320 - dist)/320, 3)/numfireballs*(wizard[n].fear)*2000;
							}
						}
						else { //consider fireball x position:
							x += Sgn(wizard[n].x - fireball[m].x) * pow((320 - dist)/320, 3)/numfireballs*(wizard[n].fear)*2000;
						}

						if ((ydist < forethought)&&(fireball[m].speed.y != 0)) {
							if ((xdist > ydist) || (ydist < xdist)) { //consider fireball y movement:
								y -= Sgn(fireball[m].speed.y) * pow((320 - dist)/320, 3)/numfireballs*(wizard[n].fear)*2000;
							}
							else { //consider fireball y position:
								y += Sgn(wizard[n].y - fireball[m].y) * pow((320 - dist)/320, 3)/numfireballs*(wizard[n].fear)*2000;
							}
						}
						else { //consider fireball y position:
							y += Sgn(wizard[n].y - fireball[m].y) * pow((320 - dist)/320, 3)/numfireballs*(wizard[n].fear)*2000;
						}


						fire -= pow((320 - dist),2)/100/numfireballs;
					}
				}
				else { //move towards slow fireballs
					if ((dist < 320) && (dist > 33) && (wizard[n].extra!=2)) {
						if (xdist/2 > ydist) x += Sgn(fireball[m].x - wizard[n].x)*pow((320 - dist)/320, 2)*(wizard[n].risk)*400;
						else if (ydist/2 > xdist) y += Sgn(fireball[m].y - wizard[n].y)*pow((320 - dist)/320, 3)*(wizard[n].risk)*400;
						else {
							x += Sgn(fireball[m].x - wizard[n].x)*pow((320 - dist)/320, 2)*(wizard[n].risk)*300;
							y += Sgn(fireball[m].y - wizard[n].y)*pow((320 - dist)/320, 2)*(wizard[n].risk)*300;
						}
					}
					if (faces(&wizard[n], &fireball[m])) fire += 500;
				}
			}

			// Avoid holes:
			if (arenatype == 4){
				for (Uint8 m = 0; m < 5; m++) {
					float dist = sqrt((hole[m].x-wizard[n].x)*(hole[m].x-wizard[n].x)+(hole[m].y-wizard[n].y)*(hole[m].y-wizard[n].y));
					float xdist = abs(wizard[n].x-hole[m].x);
					float ydist = abs(wizard[n].y-hole[m].y);
					float radius = 40;
					if (wizard[n].extra == 3) radius += 8;	
					if (difficulty > 1) radius += 4;
					if ((wizard[n].extra != 4)||(wizard[n].timer<30)){
						if (dist < radius) {
							x += Sgn(wizard[n].x - hole[m].x) * pow((xdist)/radius, 3)*(wizard[n].fear)*2000;
							y += Sgn(wizard[n].y - hole[m].y) * pow((ydist)/radius, 3)*(wizard[n].fear)*2000;
						}
					}
				}
			}
				
			//Move towards gem:
			if (extra.on){
				float xdist = abs(extra.x-wizard[n].x);
				float ydist = abs(extra.y-wizard[n].y);
				if (xdist/2 > ydist) x += Sgn(extra.x - wizard[n].x)*2000/numplayers*wizard[n].risk;
				else if (ydist/2 > xdist) y += Sgn(extra.y - wizard[n].y)*2000/numplayers*wizard[n].risk;
				else {
					x += Sgn(extra.x - wizard[n].x)*2000/numplayers*wizard[n].risk;
					y += Sgn(extra.y - wizard[n].y)*2000/numplayers*wizard[n].risk;
				}

			}

			for (Uint8 m = 0; m < numplayers; m++) {
				if (n != m) {
					float dist = distance(&wizard[n], &wizard[m]);
					float xdist = abs(wizard[n].x-wizard[m].x);
					float ydist = abs(wizard[n].y-wizard[m].y);

					//Move towards other wizards:

					if (wizard[m].status < 400) {
						if ((wizard[n].extra == 1)&&(difficulty > 0)) {
							if ((wizard[n].follows == m)&&(wizard[m].extra != 2)) {
								if (xdist > 6) x += Sgn(wizard[m].x - wizard[n].x) * 500*wizard[n].aggressive;
								if (ydist > 6) y += Sgn(wizard[m].y - wizard[n].y) * 500*wizard[n].aggressive;
							}
							if (faces(&wizard[n], &wizard[m])) {
								fire += (150 + 400/numplayers)*wizard[n].aggressive;
							}
						}
						else if (wizard[m].status < 300) {
							if ((wizard[m].extra != 2)&&(wizard[m].extra != 1)){
								if (xdist > 6) x += Sgn(wizard[m].x - wizard[n].x) * 300/numplayers*wizard[n].aggressive;
								if (ydist > 6) y += Sgn(wizard[m].y - wizard[n].y) * 300/numplayers*wizard[n].aggressive;
							}
							if (faces(&wizard[n], &wizard[m])) {
								fire += (150 + 400/numplayers)*wizard[n].aggressive;
							}
						}
						//Move away from Wizards that have been knocked down:
						else if (wizard[m].status < 400) {
							if (dist < 400) {
								x += Sgn(wizard[n].x - wizard[m].x) * pow((400 - dist)/400,2)*(1.5-wizard[n].aggressive)*100;
								y += Sgn(wizard[n].y - wizard[m].y) * pow((400 - dist)/400,2)*(1.5-wizard[n].aggressive)*100;
							}
						}
						//Move away from Firewizards:
						if ((wizard[m].extra == 1)&&(wizard[n].extra!=2)&&(wizard[n].extra!=1)&&(difficulty>0)) {
							float xdist = abs(wizard[n].x-wizard[m].x);
							float ydist = abs(wizard[n].y-wizard[m].y);

							if (dist < 320) {
								if (dist < 30) {
									if (abs(wizard[n].x - 320) > abs(wizard[n].y - 240)) {
										x -= Sgn(wizard[n].x - 320) * pow((320 - dist)/320, 3)/numfireballs*(wizard[n].fear)*800;
										y += Sgn(wizard[n].y - wizard[m].y) * pow((320 - dist)/320, 3)/numfireballs*(wizard[n].fear)*800;
									}
									else {
										x += Sgn(wizard[n].x - wizard[m].x) * pow((320 - dist)/320, 3)/numfireballs*(wizard[n].fear)*800;
										y -= Sgn(wizard[n].y - 240) * pow((320 - dist)/320, 3)/numfireballs*(wizard[n].fear)*800;									}
								}
								else {
									x += Sgn(wizard[n].x - wizard[m].x) * pow((320 - dist)/320, 3)/numfireballs*(wizard[n].fear)*800;
									y += Sgn(wizard[n].y - wizard[m].y) * pow((320 - dist)/320, 3)/numfireballs*(wizard[n].fear)*800;
								}
							}
						}
					}
				}
			}

			//Add random movement to idle wizards:
			x += (rand()%3 - 1)*2*wizard[n].idletime;
			y -= (rand()%3 - 1)*2*wizard[n].idletime;

			float delay = 0.3 - 0.05*difficulty;
			
			x = x*(1-delay)+wizard[n].aiprior.x*delay;
			y = y*(1-delay)+wizard[n].aiprior.y*delay;
			wizard[n].aiprior.x=x;
			wizard[n].aiprior.y=y;
				
			//Calculate actual movement
			Uint8 threshold = 50 - 10*difficulty;
			if (x > threshold*wizard[n].idle) wizard[n].control.x = 1;
			else if (x < -threshold*wizard[n].idle) wizard[n].control.x = -1;

			if (y > threshold*wizard[n].idle) wizard[n].control.y = 1;
			else if (y < -threshold*wizard[n].idle) wizard[n].control.y = -1;

			if (fire > 80-difficulty*40) wizard[n].control.fire = true;

			if (stop) { // don't move if the game is over
				wizard[n].control.x = 0;
				wizard[n].control.y = 0;
				wizard[n].control.fire = 0;
			}
		}
	}
}


void Game::Pause(void){
	bool paused = true;
	//CustomWait(500);
	//check that P isn't pressed anymore
	do {
		SDL_WaitEvent(&event);
	} while (event.type != SDL_KEYUP);
	SDL_PumpEvents();

	SDL_Rect rect, textrect;
	rect.x = screenoffset+140*screenfactor;
	rect.w = 360*screenfactor;
	rect.y = 160*screenfactor;
	rect.h = 160*screenfactor;	
	boxRGBA(screen, rect.x, rect.y, rect.x+rect.w, rect.y+rect.h, 0, 0, 0, 176);

	SDL_Color white={255, 255, 255};
	SDL_Surface *pausemessage;
	pausemessage = TTF_RenderUTF8_Blended(fonts.boldfont, _("-- Game Paused --"), white);
	textrect.w = pausemessage->w;
	textrect.h = pausemessage->h;
	textrect.x = 640*screenfactor/2 - pausemessage->w/2 + screenoffset;
	textrect.y = 480*screenfactor/2 - pausemessage->h/2 -24*screenfactor;
	SDL_BlitSurface (pausemessage, NULL, screen, &textrect);
	pausemessage = TTF_RenderUTF8_Blended(fonts.smallfont, _("'P' to resume"), white);
	textrect.w = pausemessage->w;
	textrect.h = pausemessage->h;
	textrect.x = 640*screenfactor/2 - pausemessage->w/2 + screenoffset;
	textrect.y = 480*screenfactor/2 - pausemessage->h/2;
	SDL_BlitSurface (pausemessage, NULL, screen, &textrect);
	pausemessage = TTF_RenderUTF8_Blended(fonts.smallfont, _("'X' to skip"),white);
	textrect.w = pausemessage->w;
	textrect.h = pausemessage->h;
	textrect.x = 640*screenfactor/2 - pausemessage->w/2 + screenoffset;
	textrect.y = 480*screenfactor/2 - pausemessage->h/2 + 15*screenfactor;
	SDL_BlitSurface (pausemessage, NULL, screen, &textrect);
	pausemessage = TTF_RenderUTF8_Blended(fonts.smallfont, _("'ESC' to quit"),white);
	textrect.w = pausemessage->w;
	textrect.h = pausemessage->h;
	textrect.x = 640*screenfactor/2 - pausemessage->w/2 + screenoffset;
	textrect.y = 480*screenfactor/2 - pausemessage->h/2 + 30*screenfactor;
	SDL_BlitSurface (pausemessage, NULL, screen, &textrect);	
	SDL_Flip(screen);	

	while (paused) {
		SDL_WaitEvent(&event);
		if (event.type == SDL_KEYDOWN) {
			if (event.key.keysym.sym == SDLK_p){
				paused = false;
			}
			else if (event.key.keysym.sym == SDLK_ESCAPE){
				paused = false;
				quit = true;
				stop = true;
			}
			else if (event.key.keysym.sym == SDLK_x) {
				fastforward = true;
				paused = false;
			}
		}
	}
	do {
		SDL_WaitEvent(&event);
	} while (event.type != SDL_KEYUP);
	rect.x = screenoffset;
	rect.w = 640*screenfactor;
	rect.y = 0;
	rect.h = 480*screenfactor;
	SDL_BlitSurface(arena, NULL, screen, &rect);
	SDL_Flip(screen);		
	SDL_PumpEvents();
	framerate.update();
}
void Game::Score(void) {
	for (Uint8 n = 0; n < numplayers; n++) {
		if (wizard[n].status < 400) wizard[n].points++;
		if (wizard[n].points == maxpoints) finished = true;
	}
}


void Game::ScoreBoard(TTF_Font *font, SDL_Surface *wizardicon[5], SDL_Surface *pointicon, SDL_Surface *emptypointicon) {
	if (fixedarena == false){
		counter=(counter+1)%2;
		if (counter == 0) {
			arenatype++;
			if (arenatype > 4) arenatype = 1;
			LoadArena();
		}
	}
	SDL_Surface *scalecopy;
	SDL_Color white = {255, 255, 255};
	SDL_Color blue = {0, 0, 196};
	SDL_Surface *textsurface;
	textsurface = TTF_RenderUTF8_Blended (font, _("Scores:"), white);
	SDL_Rect trect;
	trect.w = textsurface->w;
	trect.h = textsurface->h;
	trect.x = screenoffset+(640*screenfactor - trect.w)/2;
	trect.y = 32*screenfactor;
	SDL_BlitSurface (textsurface, NULL, screen, &trect);

	Uint16 boardwidth = screenfactor*(16 + wizardicon[0]->w/2 + (pointicon->w/2*1.5 + 5)*maxpoints);
	Uint16 boardx = screenoffset+(640*screenfactor - boardwidth)/2;

	for (Uint8 n = 0; n < numplayers; n++){
		scalecopy = zoomSurface(wizardicon[n], screenfactor/2, screenfactor/2, 1);
		SDL_Rect wrect;
		wrect.w = scalecopy->w;
		wrect.h = scalecopy->h;
		wrect.x = boardx;
		wrect.y = screenfactor*80 + n*(1.2*scalecopy->h);

		SDL_BlitSurface (scalecopy, NULL, screen, &wrect);

		for (Uint8 m = 0; m < maxpoints; m++) {
			scalecopy = zoomSurface(emptypointicon, screenfactor/2, screenfactor/2, 1);
			SDL_Rect prect;
			prect.w = scalecopy->w*1.5;
			prect.h = scalecopy->h;
			prect.x = (boardx + 16*screenfactor) + wrect.w + m*(prect.w);
			prect.y = wrect.y + (wrect.h - prect.h)/2;

			SDL_BlitSurface (scalecopy, NULL, screen, &prect);
		}

		for (Uint8 m = 0; m < wizard[n].points; m++) {
			scalecopy = zoomSurface(pointicon, screenfactor/2, screenfactor/2, 1);
			SDL_Rect prect;
			prect.w = scalecopy->w*1.5;
			prect.h = scalecopy->h;
			prect.x = boardx + 16*screenfactor + wrect.w + m*(prect.w);
			prect.y = wrect.y + (wrect.h - prect.h)/2;

			SDL_BlitSurface (scalecopy, NULL, screen, &prect);
		}
	}
	SDL_FreeSurface(scalecopy);
	SDL_Flip(screen);
	if (humanplayers > 0) WaitForKeypress();
	else if (!InterruptibleWait(3000)){
		stop=true;
		quit=true;
	}
}

void Game::Winner(TTF_Font *font) {


	SDL_FillRect (screen, NULL, 0x000000);
	SDL_Flip(screen);

	Sint8 winner = -1;

	for (Uint8 n = 0; n < numplayers; n++) {
		if (wizard[n].points == maxpoints){
			winner = n;
		}
	}

	if (Mix_PlayingMusic()){
		Mix_FadeOutMusic(1500);
		while(Mix_PlayingMusic()) {
			CustomWait(100);
		}
	}


	if (winner > -1){

		SDL_Rect rect, podiumrect;
		SDL_Surface *textsurface, *scalecopy, *winnerscreen, *podium[16];
		Mix_Chunk *stone, *applause;
		SDL_Color white = {255, 255, 255};

		textsurface = TTF_RenderUTF8_Blended (font, _("And the winner is..."), white);


		if (!(scalecopy = IMG_Load (PATH"podium/screen.png"))){
			cout << IMG_GetError();
			exit(1);
		}
		if (screenfactor == 2) winnerscreen = scalecopy;
		else {
			winnerscreen = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, 1);
			SDL_FreeSurface(scalecopy);
		}

		for (Sint8 f = 0; f < 16; f++){
			char filename[60];
			sprintf (filename, PATH"/podium/podium%02d.png", f+1);
			if (!(scalecopy = IMG_Load (filename))){
				cout << IMG_GetError();
				exit(1);
			}
			if (screenfactor == 2) podium[f] = scalecopy;
			else {
				podium[f] = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, 1);
				SDL_FreeSurface(scalecopy);
			}
		}

		if (playsounds){
			stone = Mix_LoadWAV(PATH"audio/stone.ogg");
			applause = Mix_LoadWAV(PATH"audio/applause.ogg");
		}

		podiumrect.x = screenoffset + 282*screenfactor;
		podiumrect.w = podium[0]->w;
		podiumrect.y = 208*screenfactor;
		podiumrect.h = podium[0]->h;

		rect.x = screenoffset;
		rect.w = 640*screenfactor;
		rect.y = 0;
		rect.h = 480*screenfactor;

		SDL_BlitSurface (winnerscreen, NULL, screen, &rect);
		SDL_BlitSurface (podium[0], NULL, screen, &podiumrect);
		SDL_Flip(screen);
		CustomWait(2000);

		rect.x = screenoffset+(640*screenfactor-textsurface->w)/2;
		rect.w = textsurface->w;
		rect.y = 32*screenfactor;
		rect.h = textsurface->h;
		SDL_BlitSurface (textsurface, NULL, screen, &rect);
		SDL_Flip(screen);
		CustomWait(2000);
		framerate.update();

		Sint16 timer = 0;
		Sint8 frame = 0;

		if (playsounds) Mix_PlayChannel(soundchannel, stone, 0);

		do {
			frame = timer/4;
			SDL_BlitSurface (podium[frame], NULL, screen, &podiumrect);
			SDL_Flip(screen);
			framerate.update();
			Uint16 n = framerate.updates+1;
			timer += n;
		} while (timer < 64);

		if (playsounds) Mix_FadeOutChannel(soundchannel, 200);

		CustomWait(500);

		if (playmusic){
			Mix_VolumeMusic(128);
			Mix_PlayMusic(gamemusic, -1);
			Mix_SetMusicPosition(0x11);
		}

		SDL_BlitSurface (podium[15], NULL, screen, &podiumrect);
		SDL_Flip(screen);
		CustomWait(2000);
		framerate.update();
		rect.x = screenoffset + 295*screenfactor;
		rect.w = wizardgfx[winner].burn[0]->w;
		rect.y = 210*screenfactor;
		rect.h = wizardgfx[winner].burn[0]->h;
		timer = 23;

		do {
			frame = timer/4;
			SDL_BlitSurface (podium[15], NULL, screen, &podiumrect);
			SDL_BlitSurface (wizardgfx[winner].burn[frame], NULL, screen, &rect);
			SDL_Flip(screen);
			framerate.update();
			Uint16 n = framerate.updates+1;
			timer -= n;
		} while (timer > 0);
		if (playsounds) Mix_PlayChannel(soundchannel, applause, 0);
		SDL_BlitSurface (podium[15], NULL, screen, &podiumrect);
		SDL_BlitSurface (wizardgfx[winner].walk[4][0], NULL, screen, &rect);
		SDL_Flip(screen);
		WaitForKeypress();

		SDL_FreeSurface(winnerscreen);
		Mix_FreeChunk(applause);
		Mix_FreeChunk(stone);

		for (Uint8 n = 0; n < 15; n++){
			SDL_FreeSurface(podium[n]);
		}
	}
}




// ******* auxiliary functions *******

void Game::WaitForKeypress(void) {
	bool keypress = false;
	Uint8 *keystate;
	while (keypress == false) {
		SDL_WaitEvent(&event);
		if (event.type == SDL_QUIT){
			exit(0);
		}
		else if ((event.type == SDL_KEYDOWN) || (event.type == SDL_JOYBUTTONDOWN) || (event.type == SDL_MOUSEBUTTONUP)) {
			keypress = true;
			keystate = SDL_GetKeyState (0);
			if (keystate[SDLK_ESCAPE]){ 	//quit when ESC is pressed
				quit = true;
				stop = true;
			}
		}
	}
}


void Game::SortWizards(void){
	for (int i = 0; i < numplayers; ++i) {
		for (int j = 0; j < numplayers - i - 1; j++) {
			if (wizardsorted[j]->y > wizardsorted[j + 1]->y) {
				Wizard *tmp = wizardsorted[j];
				wizardsorted[j] = wizardsorted[j + 1];
				wizardsorted[j + 1] = tmp;
			}
		}
	}
}


float Game::distance(Fireball *ball, Fireball *ball2) {
	float dist = sqrt((ball->x - ball2->x) * (ball->x - ball2->x) + (ball->y - ball2->y) * (ball->y - ball2->y));
	return dist;
}

float Game::distance(Wizard *wizard, Fireball *fireball) {
	float dist = sqrt((wizard->x - fireball->x) * (wizard->x - fireball->x) + (wizard->y - fireball->y) * (wizard->y - fireball->y));
	return dist;
}

float Game::distance(Wizard *wizard, Wizard *wizard2) {
	float dist = sqrt((wizard->x - wizard2->x) * (wizard->x - wizard2->x) + (wizard->y - wizard2->y) * (wizard->y - wizard2->y));
	return dist;
}

bool Game::collides(Wizard *wizard, Wizard *enemy){
	float dist = distance(wizard, enemy);
	Uint8 maxdist;
	if ((wizard->extra == 2)||(enemy->extra == 2)) maxdist = 30; //shield
	else if ((wizard->extra == 4)||(enemy->extra == 4)) maxdist = 0;
	else maxdist = 12;

	if  (dist < maxdist){
		return true;
	}
	else {
		return false;
	}
}


//calculate if wizard faces wizard
bool Game::faces(Wizard *wizard, Wizard *enemy){

	int xface, yface;

	switch (wizard->direction){

		case UP:
			xface = wizard->x;
			yface = wizard->y - 10;
			break;

		case UPRIGHT:
			xface = wizard->x + 7;
			yface = wizard->y - 7;
			break;

		case RIGHT:
			xface = wizard->x + 10;
			yface = wizard->y;
			break;


		case DOWNRIGHT:
			xface = wizard->x + 7;
			yface = wizard->y + 7;
			break;

		case DOWN:
			xface = wizard->x;
			yface = wizard->y + 10;
			break;

		case DOWNLEFT:
			xface = wizard->x - 7;
			yface = wizard->y + 7;
			break;

		case LEFT:
			xface = wizard->x - 10;
			yface = wizard->y;
			break;

		case UPLEFT:
			xface = wizard->x - 7;
			yface = wizard->y - 7;
			break;
	}

	float dist = sqrt((xface - enemy->x) * (xface - enemy->x) + (yface - enemy->y) * (yface - enemy->y));
	if  (dist < 12){
		return true;
	}
	else {
		return false;
	}
}


//calculate if wizard faces fireball
bool Game::faces(Wizard *wizard, Fireball *fireball){

	int xface, yface;

	switch (wizard->direction){

		case UP:
			xface = wizard->x;
			yface = wizard->y - 30;
			break;

		case UPRIGHT:
			xface = wizard->x + 21;
			yface = wizard->y - 21;
			break;

		case RIGHT:
			xface = wizard->x + 30;
			yface = wizard->y;
			break;


		case DOWNRIGHT:
			xface = wizard->x + 21;
			yface = wizard->y + 21;
			break;

		case DOWN:
			xface = wizard->x;
			yface = wizard->y + 30;
			break;

		case DOWNLEFT:
			xface = wizard->x - 21;
			yface = wizard->y + 21;
			break;

		case LEFT:
			xface = wizard->x - 30;
			yface = wizard->y;
			break;

		case UPLEFT:
			xface = wizard->x - 21;
			yface = wizard->y - 21;
			break;
	}

	float dist = sqrt((xface - fireball->x) * (xface - fireball->x) + (yface - fireball->y) * (yface - fireball->y));
	if  (dist < 25){
		return true;
	}
	else {
		return false;
	}
}


void Game::FadeOut(void) {

	Uint16 alpha = 0;
	SDL_Rect rect;
	rect.x = screenoffset;
	rect.w = 640*screenfactor;
	rect.y = 0;
	rect.h = 480*screenfactor;
	framerate.update();

	do{

		Uint8 *keystate;
		SDL_PollEvent(&event);
		keystate = SDL_GetKeyState (0);

		if (event.type == SDL_QUIT){
			SDL_Quit();
			exit(0);
		}

		if (keystate[SDLK_ESCAPE]){ 	//quit when ESC is pressed
			quit = true;
			stop = true;
			alpha = 200;
		}

		framerate.update();
		Uint16 n = framerate.updates+1;
		alpha += 5*n;
		if (alpha > 196) alpha = 196;
		SDL_BlitSurface (arena, NULL, screen, &rect);
		boxRGBA(screen, rect.x, rect.y, rect.x+rect.w, rect.y+rect.h, 0, 0, 0, alpha);
		SDL_Flip (screen);
	} while (alpha < 196);
}


void Game::FadeIn(void) {

	Sint16 alpha = 196;
	SDL_Rect rect;
	rect.x = screenoffset;
	rect.w = 640*screenfactor;
	rect.y = 0;
	rect.h = 480*screenfactor;
	framerate.update();

	do{

		Uint8 *keystate;
		SDL_PollEvent(&event);
		keystate = SDL_GetKeyState (0);

		if (event.type == SDL_QUIT){
			SDL_Quit();
			exit(0);
		}

		if (keystate[SDLK_ESCAPE]){ 	//quit when ESC is pressed
			quit = true;
			stop = true;
			alpha = 0;
		}

		framerate.update();
		Uint16 n = framerate.updates+1;
		alpha -= 5*n;
		//if (alpha < 0) alpha = 0;
		SDL_BlitSurface (arena, NULL, screen, &rect);
		if (alpha > 0) boxRGBA(screen, rect.x, rect.y, rect.x+rect.w, rect.y+rect.h, 0, 0, 0, alpha);
		SDL_Flip (screen);
	} while (alpha > 0);
}
