/*
    Copyright 2008 - 2011 Denis Comtesse <comtesse@users.sourceforge.net>

    This file is part of F.L.A.W.

    F.L.A.W. is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    F.L.A.W. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with F.L.A.W.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "wizard.h"
#include "fireball.h"
#include "framerate.h"

#define _(x) ( gettext(x) )

#include <SDL_mixer.h>
#include <SDL_gfxPrimitives.h>


class Game {

	public:
		Wizard wizard[5], *wizardsorted[5]; //the five wizards in the game
		WizardGFX wizardgfx[7];  //wizard animations (including power-ups)
		Fireball fireball[10];   //up to ten fireballs on the screen
		Fonts fonts;
		
		//other surfaces
		SDL_Surface *screen, *arena, *gem;
		SDL_Surface *fireballfly[10], *fireballstart[16], *fireballend[16], *fireballglow[10], *burn[6];

		FrameRateControl framerate;

    	//sounds
		Mix_Chunk *swing, *hit, *collide, *die, *invoke, *shoot, *extrasound[4+1], *newgem, *wah;
		Mix_Music *gamemusic, *titlemusic;

		//game logic variables
		Uint8 maxpoints, numplayers, humanplayers, numfireballs, numdead, difficulty, arenatype, counter;
		Uint16 fireballtimer;
		bool stop, finished, quit, playmusic, playsounds, fullscreen, gems, fastforward, hq, fixedarena, tactical;
		
		//gfx and sound variables
		Uint8 res; //screenmode (index)
		Uint16 xres, yres; //screen resolution
		Uint8 soundchannel; //cycle through channels when playing sound effects		
		float screenfactor, screenoffset; // scaling factor and x-offset (centered screen)
		
		float sine[360];
		Uint16 angle;
		
		//collectable gems
		struct {
			float x, y;
			bool enabled, newgem, on, blitted;
			SDL_Rect rect;
		} extra;

		//holes
		struct {
			float x, y;
			float radius;
		} hole[5];
		
		/*
		 * Class Member Comments:
		 * general gaming - applies to any game.
		 * game logic     - applies to flaw's game logic sytem.
		 * graphics       - applies to flaw's graphics sytem.
		 * user interface - applies to flaw's UI system.
		 */
		// Setup
		void DisplaySetup(void);          //general gaming
		void LoadGfx(void);               //general gaming
		void LoadArena(void);
		void AudioSetup(void);  			    //general gaming
		void RandomizeAI(void);           //game logic
		void SetStartPositions(void);     //game logic
		void StartMusic(void);            //general gaming

		//main loop
		void DisplayUpdate(void); //general gaming
		void PlaySounds(void);	  //general gaming
		void Events(void);        //general gaming
		void GameUpdate(void);    //general gaming
		void AI(void);            //general gaming
		void CheckState(void);    //general gaming
		void Pause(void);         //general gaming

		//end
		void Score(void); //game logic
		void ScoreBoard(TTF_Font *font, SDL_Surface *wizardicon[5], SDL_Surface *pointicon, SDL_Surface *emptypointicon);//game logic
		void Winner(TTF_Font *font);  //game logic

		//misc
		void FadeOut(void);       //graphics
		void FadeIn (void);       //graphics
		float distance(Fireball *ball, Fireball *ball2);    //game logic
		float distance(Wizard *wizard, Fireball *fireball); //game logic
		float distance(Wizard *wizard, Wizard *wizard2);    //game logic
		void WaitForKeypress(void); //user interface

	private:
		SDL_Event event;
		bool collides(Wizard *wizard, Wizard *enemy);    //game logic
		bool faces(Wizard *wizard, Wizard *enemy);       //game logic
		bool faces(Wizard *wizard, Fireball *fireball);  //game logic
		void SortWizards(void);                          //game logic
};
