/*
    Copyright 2008 - 2011 Denis Comtesse <comtesse@users.sourceforge.net>

    This file is part of F.L.A.W.

    F.L.A.W. is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    F.L.A.W. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with F.L.A.W.  If not, see <http://www.gnu.org/licenses/>.
*/
#define FLAWVERSION "1.3.2a"

#ifdef _WIN32
#	undef main
#	define PATH "data/"
#   define CONF "flaw-config"
#   define LANGUAGEPATH "data/locale/"
#   define FONTPATH "/data/fonts/"
#   define GETTEXT_PACKAGE "flaw"
#else
#	define PATH PACKAGE_DATA_DIR"/flaw/"
#   define CONF "%s/.flaw", getenv("HOME")
#   define LANGUAGEPATH LOCALEDIR
#endif

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <fstream>
#include <time.h>
#include "menu.h"
#include "game.h"

#define PI 3.14159265

int main (int argc, char *argv[])
{

	char configfile[60];
	sprintf(configfile, CONF);

	setlocale (LC_ALL, "");
	bindtextdomain (GETTEXT_PACKAGE, LANGUAGEPATH);
	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);

	// *** initialize SDL ***
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK) == -1) {
		cout << SDL_GetError();
		exit(1);
	}

	//init joysticks
	SDL_JoystickEventState (SDL_IGNORE);

	int numjoy = SDL_NumJoysticks();
	if (numjoy > 5) numjoy = 5;
	SDL_Joystick *joy[numjoy];

	for (Uint8 n = 0; n < numjoy; n++){
		joy[n] = SDL_JoystickOpen(n);
	}
	SDL_PumpEvents();

	atexit(SDL_Quit);

	SDL_Surface     *wmicon;
	wmicon = SDL_LoadBMP (PATH"misc/icon.bmp");
	SDL_WM_SetIcon (wmicon, NULL);


	if(TTF_Init()==-1) {
		cout << TTF_GetError();
		exit(2);
	}

	srand (time(NULL)); // random numbers

	Game game; //create "game" object

	for (Uint16 n=0; n < 360; n++) game.sine[n]=sin(n*PI/180);



	// ******* MENU *******
	//Setup Menus
	Menu menu (7, 40, false, true);
	menu.nameelement (0, _("Player Setup"));
	menu.nameelement (1, _("Game Options"));
	menu.nameelement (2, _("Graphics & Sound"));
	menu.nameelement (3, "");
	menu.nameelement (4, _(">>> Start Game <<<"));
	menu.nameelement (5, _("Watch Game Demo"));
	menu.nameelement (6, _("Quit"));


    menu.elementactive (3, false);

	Menu setup (7, 64, true,false);
	setup.nameelement (0, _("Number of Wizards: "));
	setup.nameelement (1, "  ");
	setup.nameelement (2, "  ");
	setup.nameelement (3, "  ");
	setup.nameelement (4, "  ");
	setup.nameelement (5, "  ");
	setup.nameelement (6, _("-- Back --"));

	setup.initoptions (0, 4);
	for (Uint8 n = 0; n < 4; n++) {
		char num[6];
		sprintf (num, " %d", n+2);
		setup.nameoption (0, n, num);
	}


	for (Uint8 n = 1 ; n <= 5; n++) {
    setup.initoptions (n, 9);
    for (Uint8 m = 1 ; m <= 5; m++) {
      char txt[28];
			sprintf (txt, _("Joystick %d"), m);
			setup.nameoption (n, m-1, txt);
			if (m > numjoy) setup.optionactive (n, m-1, false);
		}
		setup.nameoption (n, 5, _("Arrow Keys + Ctrl"));
		setup.nameoption (n, 6, _("WASD Keys + Shift"));
		setup.nameoption (n, 7, _("Numeric Keypad"));
		setup.nameoption (n, 8, _("<Computer>"));
	}

	Menu options (8, 36, false, true);
	options.nameelement (0, _("Winning Score: "));
	options.nameelement (1, _("Magic Gems: "));
	options.nameelement (2, _("Slow Down Fireballs: "));
	options.nameelement (3, "");
	options.nameelement (4, _("Arena: "));
	options.nameelement (5, _("Difficulty: "));
	options.nameelement (6, "");
	options.nameelement (7, _("-- Back --"));

    options.elementactive (3, false);
    options.elementactive (6, false);

	options.initoptions (0, 8);
	options.initoptions (1, 2);
	options.initoptions (2, 2);
	options.initoptions (4, 5);
	options.initoptions (5, 3);

	for (Uint8 n = 0 ; n <= 7; n++) {
		char txt[4];
		sprintf (txt, "%d", n+3);
		options.nameoption (0, n, txt);
	}

	// magic gems (active/inactive)
	options.nameoption (1, 0, _("no"));
	options.nameoption (1, 1, _("yes"));
	// fireball slowdown
	options.nameoption (2, 0, _("no"));
	options.nameoption (2, 1, _("yes"));
 	// AI wizard's difficulty setting
	options.nameoption (5, 0,_("easy"));
	options.nameoption (5, 1, _("normal"));
	options.nameoption (5, 2, _("hard"));
 	// Arena
	options.nameoption (4, 0, _("alternating"));
	options.nameoption (4, 1, _("STONE"));
	options.nameoption (4, 2, _("ICE"));
	options.nameoption (4, 3, _("FIRE"));
	options.nameoption (4, 4, _("RUINS"));

	Menu tech (7, 36, false, true);
	tech.nameelement (0, _("Music: "));
	tech.nameelement (1,_("Sound Effects: "));
	tech.nameelement (2, "");
	tech.nameelement (3, _("Resolution: "));
	tech.nameelement (4, _("Mode: "));
	tech.nameelement (5, "");
	tech.nameelement (6, _("-- Back --"));
    tech.elementactive (2, false);
    tech.elementactive (5, false);

	tech.initoptions (0, 2);
	tech.initoptions (1, 2);
	tech.initoptions (3, 10);
	tech.initoptions (4, 2);

	tech.nameoption (0, 0, _("off"));
	tech.nameoption (0, 1, _("on"));
	tech.nameoption (1, 0, _("off"));
	tech.nameoption (1, 1, _("on"));
	tech.nameoption (3, 0, _("320x240"));
	tech.nameoption (3, 1, _("640x400"));
	tech.nameoption (3, 2, _("640x480"));
	tech.nameoption (3, 3, _("800x600"));
	tech.nameoption (3, 4, _("1280x720"));
	tech.nameoption (3, 5, _("1024x768"));
	tech.nameoption (3, 6, _("1366x768"));
	tech.nameoption (3, 7, _("1280x800"));
	tech.nameoption (3, 8, _("1280x960"));
	tech.nameoption (3, 9, _("1280x1024"));
	tech.nameoption (4, 0, _("Window"));
	tech.nameoption (4, 1, _("Fullscreen"));

	if (!SDL_VideoModeOK(1280, 1024, 16, SDL_SWSURFACE |  SDL_FULLSCREEN)) tech.optionactive(3, 9, false);
	if (!SDL_VideoModeOK(1280, 960, 16, SDL_SWSURFACE |  SDL_FULLSCREEN)) tech.optionactive(3, 8, false);
	if (!SDL_VideoModeOK(1280, 800, 16, SDL_SWSURFACE |  SDL_FULLSCREEN)) tech.optionactive(3, 7, false);
	if (!SDL_VideoModeOK(1366, 768, 16, SDL_SWSURFACE |  SDL_FULLSCREEN)) tech.optionactive(3, 6, false);
	if (!SDL_VideoModeOK(1280, 720, 16, SDL_SWSURFACE |  SDL_FULLSCREEN)) tech.optionactive(3, 4, false);
	if (!SDL_VideoModeOK(320, 240, 16, SDL_SWSURFACE |  SDL_FULLSCREEN)) tech.optionactive(3, 0, false);
	if (!SDL_VideoModeOK(640, 400, 16, SDL_SWSURFACE |  SDL_FULLSCREEN)) tech.optionactive(3, 1, false);
	if (!SDL_VideoModeOK(640, 480, 16, SDL_SWSURFACE |  SDL_FULLSCREEN)) tech.optionactive(3, 2, false);

	//load options:
	ifstream file(configfile);
	char x[8];
	file >> x;
	bool reset = false;

	for (Uint8 n = 1; n < argc; n++){
		if ((strcmp(argv[n],"-r") == 0) || (strcmp(argv[n],"--reset") == 0)) reset = true;
	}

	if ((!file.fail())&&(atoi(x) == 131)&&(!reset)) {

		for (Uint8 n=0; n<6; n++){
			file >> x;
			setup.setoption (n,atoi(x));
		}

		file >> x;
		options.setoption (0,atoi(x));
		file >> x;
		options.setoption (1,atoi(x));
		file >> x;
		options.setoption (2,atoi(x));
		file >> x;
		options.setoption (4,atoi(x));
		file >> x;
		options.setoption (5,atoi(x));
		file >> x;
		tech.setoption (0,atoi(x));
		file >> x;
		tech.setoption (1,atoi(x));
		file >> x;
		tech.setoption (3,atoi(x));
		file >> x;
		tech.setoption (4,atoi(x));
	}

	else {
		setup.setoption(0, 3);
		setup.setoption(1, 5);
		setup.setoption(2, 8);
		setup.setoption(3, 8);
		setup.setoption(4, 8);
		setup.setoption(5, 8);
		options.setoption (0, 2);
		options.setoption (1, 1);
		options.setoption (2, 0);
		options.setoption (4, 1);
		options.setoption (5, 1);
		tech.setoption (0, 1);
		tech.setoption (1, 1);
		tech.setoption (3, 3);
		tech.setoption (4, 0);
	}
	file.close();

	for (Uint8 n = 1; n < argc; n++){
		if ((strcmp(argv[n],"-f") == 0) || (strcmp(argv[n],"--fullscreen") == 0)) tech.setoption (4, 1);
	}

	for (Uint8 n = 1; n < argc; n++){
		if ((strcmp(argv[n],"-w") == 0) || (strcmp(argv[n],"--windowed") == 0)) tech.setoption (4, 0);
	}

    if (tech.getoption(4)) game.fullscreen = true;
    else game.fullscreen = false;

	game.res = tech.getoption(3);
	game.hq = tech.getoption(5);

	game.DisplaySetup();
	float fac = game.screenfactor;
	float offset = game.screenoffset;

	// Load titlescreen, menu icons, fonts:
	SDL_Surface *wizardicon[5], *pointicon, *emptypointicon, *noicon, *title;

	if (!(title = IMG_Load (PATH"misc/menubackground.png"))){
		cout << IMG_GetError();
	exit(1);
	}

	if (!(wizardicon[0] = IMG_Load (PATH"misc/wizard_blue.png"))){
		cout << IMG_GetError();
		exit(1);
	}
	if (!(wizardicon[1] = IMG_Load (PATH"misc/wizard_red.png"))){
		cout << IMG_GetError();
		exit(1);
	}
	if (!(wizardicon[2] = IMG_Load (PATH"misc/wizard_green.png"))){
		cout << IMG_GetError();
		exit(1);
	}
	if (!(wizardicon[3] = IMG_Load (PATH"misc/wizard_black.png"))){
		cout << IMG_GetError();
		exit(1);
	}
	if (!(wizardicon[4] = IMG_Load (PATH"misc/wizard_purple.png"))){
		cout << IMG_GetError();
		exit(1);
	}


	if (!(pointicon = IMG_Load (PATH"misc/point.png"))){
		cout << IMG_GetError();
		exit(1);
	}
	if (!(emptypointicon = IMG_Load (PATH"misc/nopoint.png"))){
		cout << IMG_GetError();
		exit(1);
	}

	if (!(noicon = IMG_Load (PATH"misc/blank.png"))){
		cout << IMG_GetError();
		exit(1);
	}

	if (!(game.fonts.LoadFonts(FONTPATH"", setlocale(LC_ALL, NULL), fac))){
		cerr << "Couldn't load fonts!";
		exit(1);
	}

	menu.setfonts (game.fonts.normalfont, game.fonts.boldfont);
	setup.setfonts (game.fonts.normalfont, game.fonts.boldfont);
	options.setfonts (game.fonts.normalfont, game.fonts.boldfont);
	tech.setfonts (game.fonts.normalfont, game.fonts.boldfont);

	setup.seticon (0, noicon);
	setup.seticon (1, wizardicon[0]);
	setup.seticon (2, wizardicon[1]);
	setup.seticon (3, wizardicon[2]);
	setup.seticon (4, wizardicon[3]);
	setup.seticon (5, wizardicon[4]);
	setup.seticon (6, noicon);



	// Game Setup
	game.AudioSetup();

	if (numjoy > 0) {
		menu.setjoystick(joy[0]);
		setup.setjoystick(joy[0]);
		options.setjoystick(joy[0]);
		tech.setjoystick(joy[0]);
		SDL_JoystickEventState (SDL_ENABLE);
	}

	bool reload = true; //load graphics on first game start


	//the big loop:

	do{

		bool demo = false;
		float fac = game.screenfactor;
		float offset = game.screenoffset;

		SDL_ShowCursor(SDL_ENABLE);
		SDL_WarpMouse(offset+640*fac, 480*fac);

		if (tech.getoption(0)){
			Mix_VolumeMusic(128);
			Mix_PlayMusic(game.titlemusic, -1);
			Mix_SetMusicPosition(0x01);
		}
		else{
			Mix_FadeOutMusic(100);
		}

		bool done = false;

		//MAIN MENU:
		while (!done) {
			Sint8 selected;
			menu.select(0);
			SDL_Surface *screen_scaled;
			SDL_Surface *textsurface;
			SDL_Rect textrect, titlerect;
			SDL_Color orange = {220,120,0};
			SDL_Color black = {0,0,0};
			if (fac < 0.8) textsurface = TTF_RenderUTF8_Blended (game.fonts.tinyfont, FLAWVERSION, orange);
			else textsurface = TTF_RenderUTF8_Blended (game.fonts.smallfont, FLAWVERSION, orange);
			textrect.x = offset;
			textrect.y = 480*fac-textsurface->h;
			textrect.w = textsurface->w;
			textrect.h = textsurface->h;
			screen_scaled = zoomSurface (title, fac/2, fac/2, 1);
			titlerect.x = offset;
			titlerect.y = 0;
			titlerect.w = screen_scaled->w;
			titlerect.h = screen_scaled->h;
			SDL_BlitSurface (screen_scaled, NULL, game.screen, &titlerect);
			SDL_BlitSurface (textsurface, NULL, game.screen, &textrect);
			menu.setspacing (40*fac);
			menu.screenfactor = game.screenfactor;
			selected = menu.run(offset, 130*fac, 640*fac, 300*fac);

			//Exit:
			if (selected == 6){

				//save options:
				ofstream file(configfile, ios::trunc);
				if(!file.fail())
				{
					file << "131" << endl;
					char x[2];
					for (Uint8 n=0; n<6; n++){
						sprintf (x, "%d", setup.getoption (n));
						file << x << endl;
					}

					sprintf (x, "%d", options.getoption (0));
					file << x << endl;
					sprintf (x, "%d", options.getoption (1));
					file << x << endl;
					sprintf (x, "%d", options.getoption (2));
					file << x << endl;
					sprintf (x, "%d", options.getoption (4));
					file << x << endl;
					sprintf (x, "%d", options.getoption (5));
					file << x << endl;
					sprintf (x, "%d", tech.getoption (0));
					file << x << endl;
					sprintf (x, "%d", tech.getoption (1));
					file << x << endl;
					sprintf (x, "%d", tech.getoption (3));
					file << x << endl;
					sprintf (x, "%d", tech.getoption (4));
					file << x << endl;
					file.close();
				}
				else cerr << "Could not create config file!";
				exit(0);
			}

			//Start the Game:
			else if (selected == 4) {
				done = true;
				demo = false;
			}

			/*//Show Instructions
			else if (selected == 3) {
				SDL_Surface *instructions, *instructions_scaled;
				SDL_Rect instrect;
				if (!(instructions = IMG_Load (PATH"misc/instructions.png"))){
					cout << IMG_GetError();
				exit(1);
				}
				if (fac != 2) {
					instructions_scaled = zoomSurface (instructions, fac/2, fac/2, 1);
				}
				else {
					instructions_scaled = instructions;
				}

				instrect.x = offset;
				instrect.y = 0;
				instrect.w = instructions_scaled->w;
				instrect.h = instructions_scaled->h;

				SDL_BlitSurface (instructions_scaled, NULL, game.screen, &instrect);
				SDL_Flip (game.screen);
				game.WaitForKeypress();
			}*/

			// Demo Game
			else if (selected == 5) {
				done = true;
				demo = true;
			}


			//Player Setup:
			else if (selected == 0) {

				setup.select(6);

				//Update Joysticks before Setup
				SDL_JoystickEventState (SDL_IGNORE);
				SDL_PumpEvents();
				SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
				SDL_InitSubSystem(SDL_INIT_JOYSTICK);

				int numjoy = SDL_NumJoysticks();
				if (numjoy > 5) numjoy = 5;

				for (Uint8 n = 0; n < numjoy; n++){
					joy[n] = SDL_JoystickOpen(n);
				}

				if (numjoy > 0){
					menu.setjoystick(joy[0]);
					setup.setjoystick(joy[0]);
					options.setjoystick(joy[0]);
					tech.setjoystick(joy[0]);
					SDL_PumpEvents();
					SDL_JoystickEventState (SDL_ENABLE);
				}
				else{
					menu.nojoystick();
					setup.nojoystick();
					options.nojoystick();
					tech.nojoystick();
				}


				for (Uint8 n = 1 ; n <= 5; n++) {
					for (Uint8 m = 1 ; m <= 5; m++) {
						if (m > numjoy){
							setup.optionactive (n, m-1, false);
						}
						else {
							setup.optionactive (n, m-1, true);
						}
					}
					if ((setup.getoption(n) >= numjoy) && (setup.getoption(n) < 5)) setup.setoption (n, 6);
				}

				setup.screenfactor = game.screenfactor;
				setup.setspacing (48*fac);

				do {
                    for (Uint8 n = 2; n < 6; n++) {
                        if (n - 2 <= setup.getoption (0)) {
                            setup.elementactive (n, true);
                        }
                        else {
                            setup.elementactive (n, false);
                        }
                    }
					SDL_BlitSurface (screen_scaled, NULL, game.screen, &titlerect);
					SDL_BlitSurface (textsurface, NULL, game.screen, &textrect);
					selected = setup.run(offset+180*fac, 110*fac, 640*fac, 340*fac);
				}while ((selected != 6)&&(selected != -1));
			}

			//Game Options:
			else if (selected == 1) {

				options.select(7);
				options.setspacing (40*fac);
				options.screenfactor = game.screenfactor;
				do {
					SDL_BlitSurface (screen_scaled, NULL, game.screen, &titlerect);
					SDL_BlitSurface (textsurface, NULL, game.screen, &textrect);
					selected = options.run(offset, 120*fac, 640*fac, 316*fac);
				}while ((selected != 7)&&(selected != -1));
			}

			//GFX and Sound Options:
			else if (selected == 2) {

				tech.select(6);

				Uint8 res = tech.getoption(3);
				Uint8 fs = tech.getoption(4);
				//Uint8 hq = tech.getoption(5);

				tech.setspacing (36*fac);
				tech.screenfactor = game.screenfactor;
				do {
					SDL_BlitSurface (screen_scaled, NULL, game.screen, &titlerect);
					SDL_BlitSurface (textsurface, NULL, game.screen, &textrect);
					selected = tech.run(offset, 120*fac, 640*fac, 340*fac);
				}while ((selected != 6)&&(selected != -1));

				if (tech.getoption(4) == 1) game.fullscreen = true;
				else game.fullscreen = false;
				game.res = tech.getoption(3);
				//game.hq = tech.getoption(5);

				if ((tech.getoption(0))&&(!Mix_PlayingMusic())){
					Mix_VolumeMusic(128);
					Mix_PlayMusic(game.titlemusic, -1);
					Mix_SetMusicPosition(0x01);
				}
				else if ((!tech.getoption(0))&&(Mix_PlayingMusic())){
					Mix_FadeOutMusic(1500);
					while(Mix_PlayingMusic()) {
						SDL_Delay(100);
					}
				}

				if (res != game.res) {
					game.DisplaySetup();
					reload = true;

					fac = game.screenfactor;
					offset = game.screenoffset;
					if (!(game.fonts.LoadFonts(FONTPATH"", setlocale(LC_ALL, NULL), fac))){
						cerr << "Couldn't load fonts!";
						exit(1);
					}
					menu.setfonts (game.fonts.normalfont, game.fonts.boldfont);
					setup.setfonts (game.fonts.normalfont, game.fonts.boldfont);
					options.setfonts (game.fonts.normalfont, game.fonts.boldfont);
					tech.setfonts (game.fonts.normalfont, game.fonts.boldfont);
				}
				else if (fs != tech.getoption(4)) game.DisplaySetup();

			}
		}


		//NEW GAME:


		//settings:

		if (options.getoption(4) == 0){
			game.fixedarena = false;
			game.arenatype = 1;
		}
		else {
			game.fixedarena = true;
			game.arenatype = options.getoption(4);
		}

		if (tech.getoption(4)) SDL_ShowCursor(SDL_DISABLE);

		SDL_FillRect (game.screen, NULL, 0x000000);
		SDL_Flip(game.screen);

		if (reload) { // reload and rescale graphics if resolution was changed
			SDL_Surface *waitingmessage;
			SDL_Rect rect;
			SDL_Color white = {255, 255, 255};
			waitingmessage = TTF_RenderUTF8_Blended (game.fonts.boldfont, _("Loading..."), white);
			rect.w = waitingmessage->w;
			rect.h = waitingmessage->h;
			rect.x = 640*fac/2 - waitingmessage->w/2 + offset;
			rect.y = 480*fac/2 - waitingmessage->h/2;
			SDL_BlitSurface (waitingmessage, NULL, game.screen, &rect);
			SDL_Flip(game.screen);

			game.LoadGfx();
			game.LoadArena();
			reload = false;
		}

		game.numplayers = setup.getoption(0) + 2;
		game.maxpoints = options.getoption(0) + 3;
		game.gems = options.getoption(1);
		game.tactical = options.getoption(2);
		game.difficulty = options.getoption(5);

		game.framerate.init(); // adjust game speed to difficulty settings
		if (options.getoption(5) == 0) 	game.framerate.setfps (45);
		else if (options.getoption(5) == 1) game.framerate.setfps (50);
		else game.framerate.setfps (55);

		if (tech.getoption(0)) game.playmusic = true;
		else game.playmusic = false;
		if (tech.getoption(1)) game.playsounds = true;
		else game.playsounds = false;


		if (demo == true) {
			game.fixedarena = true;
			game. arenatype = 1;
			game.numplayers = 5;
			game.maxpoints = 3;
			game.gems = true;
			game.tactical = false;
			game.difficulty = 1;
			game.framerate.setfps(50);
		}

		game.humanplayers = 0;

		for (Uint8 n = 0; n < game.numplayers; n++) {
			Uint8 option = setup.getoption(n+1);
			if (demo == true){
				game.wizard[n].keyboard = false;
				game.wizard[n].ai = true;
				game.wizard[n].joystick = false;
				game.wizard[n].points = 0;
			}
			else if (option == 5) {
				game.wizard[n].keyboard = 1;
				game.wizard[n].ai = false;
				game.wizard[n].joystick = false;
				game.wizard[n].points = 0;
				game.humanplayers++;
			}
			else if (option == 6) {
				game.wizard[n].keyboard = 2;
				game.wizard[n].ai = false;
				game.wizard[n].joystick = false;
				game.wizard[n].points = 0;
				game.humanplayers++;
			}
			else if (option == 7) {
				game.wizard[n].keyboard = 3;
				game.wizard[n].ai = false;
				game.wizard[n].joystick = false;
				game.wizard[n].points = 0;
				game.humanplayers++;
			}
			else if (option == 8) {
				game.wizard[n].keyboard = false;
				game.wizard[n].ai = true;
				game.wizard[n].joystick = false;
				game.wizard[n].points = 0;
			}
			else {
				game.wizard[n].keyboard = false;
				game.wizard[n].ai = false;
				game.wizard[n].joystick = true;
				game.wizard[n].joy = joy[option];
				game.wizard[n].points = 0;
				game.humanplayers++;
			}
		}


		// ******* Start Game *******

		game.LoadArena();
		game.counter = 0;
		game.RandomizeAI();
		game.quit = false;
		game.finished = false;

		if (Mix_PlayingMusic()){
			Mix_FadeOutMusic(800);
			while(Mix_PlayingMusic()) {
				SDL_Delay(100);
			}
		}

		game.StartMusic();

		while ((!game.quit) && (!game.finished)){

			game.SetStartPositions();
			game.fastforward = false;
			game.numdead = 0;
			game.fireballtimer = game.framerate.framespersecond*6;
			game.stop = false;
			if (!game.quit) game.FadeIn();
			game.framerate.update();

			//game loop
			Uint8 timer = 0; // timer: short delay when the game ends -> finish animations
			while ((!game.quit)&&(timer < 80)) {
				Sint16 numupdates;
				if (!game.fastforward){  //fastforward -> skip the rest of the match and just show the result
					game.DisplayUpdate();
					game.Events();
					game.framerate.update();
					numupdates = game.framerate.updates;
				}
				else numupdates = 0;

				if (game.stop) timer += numupdates+1;
				for (numupdates; numupdates >= 0; numupdates--){
					game.AI();
					game.GameUpdate();
					if ((game.playsounds)&&(!game.fastforward)) game.PlaySounds();
				}
			}

			if (!game.quit) game.Score();
			if (!game.quit) game.FadeOut();
			if (!game.quit) game.ScoreBoard(game.fonts.boldfont, wizardicon, pointicon, emptypointicon);
		}
		game.Winner(game.fonts.boldfont);

	} while (true);

}
