/*
    Copyright 2008 - 2011 Denis Comtesse <comtesse@users.sourceforge.net>

    This file is part of F.L.A.W.

    F.L.A.W. is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    F.L.A.W. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with F.L.A.W.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "menu.h"


Menu::Menu (Uint8 n, Uint16 s, bool i, bool c) {

	if (numelements <= 20) numelements = n;
	else numelements = 20;
	spacing = s;
	useicons = i;
	centered = c;
	selected = 0;
	usejoystick = false;

	for (Uint8 e = 0; e < 20; e++) {
		if (e < n) element[e].active = true;
		else element[e].active = false;
		element[e].numoptions = 0;
		element[e].state = 0;
		element[e].switchable = false;
		for (Uint8 o = 0; o < 20; o++) {
			element[e].option[o].active = false;
		}
	}
}

void Menu::elementactive (Uint8 e, bool b) {
	if (e < numelements){
		element[e].active = b;
	}
	else cout << "Menu::elementactive: Invalid Element";
}

void Menu::optionactive (Uint8 e, Uint8 o, bool b) {
	if ((e < numelements) && (o < element[e].numoptions)){
		element[e].option[o].active = b;
	}
	else cerr << "Menu::optionactive: Wrong values!";
}

void Menu::initoptions (Uint8 e, Uint8 n) {
	if ((e < numelements) && (n <= 20)) {
		element[e].switchable = true;
		element[e].numoptions = n;
		for (Uint8 o = 0; o < n; o++){
			element[e].option[o].active = true;
		}
	}
	else cerr << "Menu::initoptions: Wrong values!";
}

void Menu::nameelement(Uint8 e, const char *n) {
	if ((e < numelements) && (strlen(n) <= 70)) {
		sprintf (element[e].name, "%s", n);
	}
	else cerr << "Menu::namelement: Invalid Element or string too long!";
}

void Menu::nameoption(Uint8 e, Uint8 o, const char *n) {
	if ((e < numelements) && (o < element[e].numoptions) && (strlen(n) <= 70)) {
		sprintf (element[e].option[o].name, "%s", n);
	}
	else cout << "Menu::nameoption: Wrong values or string too long!";
}

void Menu::setfonts(TTF_Font *normalfont, TTF_Font *boldfont) {
	normal = normalfont;
	bold = boldfont;
}

void Menu::setspacing (Uint16 s) {
	spacing = s;
}


void Menu::seticon(Uint8 e, SDL_Surface *s) {
	if (e < numelements){
		element[e].icon = s;
	}
	else cout << "Menu::seticon: Invalid Element";
}

void Menu::setoption (Uint8 e, Uint8 o) {
	if (e < numelements) {
		if (element[e].option[o].active == true) {
			element[e].state = o;
		}
		else cout << "Menu::setoption: Invalid Option (out of range or inactive)";
	}
	else cout << "Menu::seticons: Invalid Element";
}

Uint8 Menu::getoption (Uint8 e) {
	if (e < numelements) {
		return element[e].state;
	}
	else {
		cout << "Menu::seticons: Invalid Element";
		return 0;
	}
}

void Menu::select (Uint8 e) {
	if (e < numelements) {
		selected = e;
	}
	else {
		cout << "Menu::select: Invalid Element";
	}
}

void Menu::setjoystick (SDL_Joystick *j) {
	if (!j) {
		cout << "Menu::setjoystick: No Joystick?";
	}
	else {
		joy = j;
		usejoystick = true;
	}
}

void Menu::nojoystick (void) {
	usejoystick = false;
}


Sint8 Menu::run (Uint16 x, Uint16 y, Uint16 w, Uint16 h) {

	SDL_Surface *s = SDL_GetVideoSurface();

	SDL_Color colour_active = {255, 255, 255};
	SDL_Color colour_inactive = {128,128,128};
	SDL_Color colour_shadow = {0,0,0};
	SDL_Surface *background;
	background=SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, 0, 0, 0, 0);
	SDL_Rect rect, buttonrect[numelements], backgroundrect;
	rect.x = x;
	rect.w = w;
	rect.y = y;
	rect.h = h;
	backgroundrect = rect;

	SDL_BlitSurface (s, &backgroundrect, background, NULL);
	SDL_Event event, empty_event;

	//check that no button is pressed
	do {
		SDL_PollEvent(&event);
	} while ((SDL_JoystickGetButton(joy, 0)) || (SDL_JoystickGetButton(joy, 1)));


	SDL_PumpEvents();
	bool done = false;
	bool leave = false;
	while (done == false) {
		SDL_BlitSurface (background, NULL, s, &backgroundrect);
		//SDL_FillRect (s, &rect, 0x000000);
		for (Uint8 e = 0; e < numelements; e++) {
			if (element[e].active) {
				char txt[150];
				if (element[e].option[element[e].state].active) {
					sprintf (txt, "%s %s", element[e].name, element[e].option[element[e].state].name);
				}
				else {
					sprintf (txt, "%s", element[e].name);
				}

				SDL_Surface *textsurface, *textshadowsurface;
				SDL_Rect rect, shadowrect, iconrect;
				if (selected == e) {
					textsurface = TTF_RenderUTF8_Blended (bold, txt, colour_active);
					textshadowsurface = TTF_RenderUTF8_Blended (bold, txt, colour_shadow);
				}
				else {
					textsurface = TTF_RenderUTF8_Blended (normal, txt, colour_inactive);
					textshadowsurface = TTF_RenderUTF8_Blended (normal, txt, colour_shadow);
				}

				rect.w = textsurface->w;
				rect.h = textsurface->h;
				rect.y = y + e*spacing;

				if (!centered){
					rect.x = x;
					iconrect.x = x;
				}
				else {
					rect.x = x + (640*screenfactor - textsurface->w)/2;
					iconrect.x = x + (640*screenfactor - textsurface->w)/2;
				}

				shadowrect = rect;
				shadowrect.x = rect.x + rect.h/12;
				shadowrect.y = rect.y + rect.h/12;		
				

				if (useicons){
					SDL_Surface *scalecopy;
					if (screenfactor == 2) scalecopy = element[e].icon;
					else scalecopy = zoomSurface(element[e].icon, screenfactor/2, screenfactor/2, 1);

					if (!centered) {
						rect.x += scalecopy->w;
						shadowrect.x += scalecopy->w;
					}
					else {
						rect.x += scalecopy->w/2;
						shadowrect.x += scalecopy->w/2;
						iconrect.x -= scalecopy->w/2;
					}
					rect.y += (scalecopy->h - textsurface->h)/2;
					shadowrect.y += (scalecopy->h - textsurface->h)/2;
					iconrect.w = textsurface->w;
					iconrect.h = textsurface->h;
					iconrect.y = y + e*spacing;
					SDL_BlitSurface (scalecopy, NULL, s, &iconrect);
					SDL_BlitSurface (textshadowsurface, NULL, s, &shadowrect);
					SDL_BlitSurface (textsurface, NULL, s, &rect);

				}
				else{
					SDL_BlitSurface (textsurface, NULL, s, &rect);
				}

				buttonrect[e] = rect;
				SDL_BlitSurface (textshadowsurface, NULL, s, &shadowrect);
				SDL_BlitSurface (textsurface, NULL, s, &rect);

			}
		}
		SDL_Flip (s);
		bool change = false;

		while (!change){
			SDL_PollEvent(&event);
			if (event.type == SDL_MOUSEMOTION){
				int mousex, mousey;
				SDL_GetMouseState(&mousex, &mousey);
				for (Uint8 e = 0; e < numelements; e++) {
					if ((element[e].active) && (mousex >= buttonrect[e].x) && (mousex <= buttonrect[e].x+buttonrect[e].w)){
						if ((mousey >= buttonrect[e].y) && (mousey <= buttonrect[e].y+buttonrect[e].h)){
							if (selected != e) {
								selected = e;
								change = true;
							}
						}
					}
				}
			}


			if (event.type == SDL_QUIT){
				SDL_Quit();
				exit(0);
			}

			if (event.type == SDL_KEYDOWN) {
				if (event.key.keysym.sym == SDLK_ESCAPE) {
					leave = true;
					done = true;
					change = true;
				}

				else if (event.key.keysym.sym == SDLK_UP) {
					do {
						if (selected == 0) selected = numelements-1;
						else selected -= 1;
					} while (!element[selected].active);
					change = true;
				}

				else if (event.key.keysym.sym == SDLK_DOWN) {
					do {
						selected = (selected + 1) % numelements;
					} while (!element[selected].active);
					change = true;
				}

				else if ((event.key.keysym.sym == SDLK_RCTRL) || (event.key.keysym.sym == SDLK_RETURN) || (event.key.keysym.sym == SDLK_RSHIFT)) {

					if (element[selected].switchable) {
						do {
							element[selected].state = (element[selected].state + 1) % element[selected].numoptions;
						} while (!element[selected].option[element[selected].state].active);
					}
					change = true;
					done = true;
				}
				SDL_PushEvent(&empty_event);
			}


			if (event.type == SDL_MOUSEBUTTONUP) {
				if (event.button.button == SDL_BUTTON_RIGHT) {
					leave = true;
					done = true;
					change = true;
				}
				else if (event.button.button == SDL_BUTTON_LEFT){
					if (element[selected].switchable) {
						do {
							element[selected].state = (element[selected].state + 1) % element[selected].numoptions;
						} while (!element[selected].option[element[selected].state].active);
					}
					change = true;
					done = true;
					SDL_PushEvent(&empty_event);
				}
			}


			if (usejoystick){
				if (event.type == SDL_JOYAXISMOTION) {

					if ((event.jaxis.which == 0)&&(event.jaxis.axis == 1)&&(event.jaxis.value < -8000)) {
						do {
							if (selected == 0) selected = numelements-1;
							else selected -= 1;
						} while (!element[selected].active);
						change = true;
					}

					else if ((event.jaxis.which == 0)&&(event.jaxis.axis == 1)&&(event.jaxis.value > 8000)) {
						do {
							selected = (selected + 1) % numelements;
						} while (!element[selected].active);
						change = true;
					}
					SDL_PushEvent(&empty_event);
				}

				else if (event.type == SDL_JOYBUTTONDOWN){
					if ((event.jbutton.button < 2)&&(event.jbutton.state == SDL_PRESSED)){
						if (element[selected].switchable) {
							do {
								element[selected].state = (element[selected].state + 1) % element[selected].numoptions;
							} while (!element[selected].option[element[selected].state].active);
						}
						change = true;
						done = true;
						SDL_PushEvent(&empty_event);
					}
				}
			}
		}
	}
	if (leave == true) return -1;
	else return selected;

}