/*
    Copyright 2008 - 2011 Denis Comtesse <comtesse@users.sourceforge.net>

    This file is part of F.L.A.W.

    F.L.A.W. is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    F.L.A.W. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with F.L.A.W.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "misc.h"
#include <iostream>
#include <iterator>
#include <fstream>
#include <dirent.h>

using namespace std;

Uint32 GetPixel(SDL_Surface *surface, int x, int y)
{
    int bpp = surface->format->BytesPerPixel;
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        return *p;
        break;

    case 2:
        return *(Uint16 *)p;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;
        break;

    case 4:
        return *(Uint32 *)p;
        break;

    default:
        return 0;
    }
}

//Replacement for SDL_Delay which causes problems with screensavers
void CustomWait(Uint32 ms) {
	SDL_Event event;
    Uint32 startticks, ticks;
    startticks = SDL_GetTicks();
    do {
        SDL_PollEvent(&event); //do something
        ticks = SDL_GetTicks();
    } while (ticks - startticks < ms);
}

bool InterruptibleWait(Uint32 ms) {
	bool r = true;
	SDL_Event event;
    Uint32 startticks, ticks;
	startticks = SDL_GetTicks();
	bool keypress = false;
	Uint8 *keystate;
	do {
		SDL_PollEvent(&event);
		ticks = SDL_GetTicks();
		if (event.type == SDL_QUIT){
			exit(0);
		}
		else if ((event.type == SDL_KEYDOWN) || (event.type == SDL_JOYBUTTONDOWN) || (event.type == SDL_MOUSEBUTTONUP)) {
			keypress = true;
			keystate = SDL_GetKeyState (0);
			if (keystate[SDLK_ESCAPE]){ 	//quit when ESC is pressed
				r = false;
			}
		}

	} while ((ticks - startticks < ms)&&(keypress == false)) ;
	return r;
}




int randomnumber(int a, int b) {
	double r = b - a + 1;
	return a + (int)(r * rand()/(RAND_MAX+1.0));
}

Sint8 Sgn(float a){
	Sint8 s;
	if (a < 0){
		s = -1;
	}
	else if (a > 0){
		s = 1;
	}
	else{
		s = 0;
	}
	return s;
}


#ifdef _WIN32

bool Fonts::LoadFonts(const char *path, const char *lc, float fac) {
	if ((strncmp(lc,"zh",2)==0)||(strncmp(lc,"Chin",4)==0)||(strncmp(lc,"ja",2)==0)||(strncmp(lc,"Japan",5)==0)){
		if (!(normalfont = TTF_OpenFont("data/fonts/wqy-microhei/wqy-microhei.ttc", 24*fac))) return false;
		if (!(boldfont = TTF_OpenFont("data/fonts/wqy-microhei/wqy-microhei.ttc", 26*fac))) return false;
		if (!(smallfont = TTF_OpenFont("data/fonts/wqy-microhei/wqy-microhei.ttc", 12*fac))) return false;
		if (!(tinyfont = TTF_OpenFont("data/fonts/wqy-microhei/wqy-microhei.ttc", 9*fac))) return false;
		return true;
	}
	if (!(normalfont = TTF_OpenFont("data/fonts/freefont/FreeSerif.ttf", 24*fac))) return false;
	if (!(boldfont = TTF_OpenFont("data/fonts/freefont/FreeSerifBold.ttf", 24*fac))) return false;
	if (!(smallfont = TTF_OpenFont("data/fonts/freefont/FreeSans.ttf", 12*fac))) return false;
	if (!(tinyfont = TTF_OpenFont("data/fonts/freefont/FreeSans.ttf", 9*fac))) return false;
	return true;
}

#else

bool Fonts::LoadFonts(const char *path, const char *lc, float fac) {
	char foundpath[200];
	if ((strncmp(lc,"zh",2)==0)||(strncmp(lc,"Chin",4)==0)||(strncmp(lc,"ja",2)==0)||(strncmp(lc,"Japan",5)==0)){
		FindFile (path, "wqy-microhei.ttc", foundpath);
		if (!strcmp(foundpath,"none")) FindFile (path, "wqy-zenhei.ttc", foundpath);
		if (!(normalfont = TTF_OpenFont(foundpath, 24*fac))) return false;
		if (!(boldfont = TTF_OpenFont(foundpath, 26*fac))) return false;
		if (!(smallfont = TTF_OpenFont(foundpath, 12*fac))) return false;
		if (!(tinyfont = TTF_OpenFont(foundpath, 9*fac))) return false;
		return true;
	}
	FindFile (path, "FreeSerif.ttf", foundpath);
	if (!(normalfont = TTF_OpenFont(foundpath, 24*fac))) return false;
	FindFile (path, "FreeSerifBold.ttf", foundpath);
	if (!(boldfont = TTF_OpenFont(foundpath, 24*fac))) return false;
	FindFile (path, "FreeSans.ttf", foundpath);
	if (!(smallfont = TTF_OpenFont(foundpath, 12*fac))) return false;
	if (!(tinyfont = TTF_OpenFont(foundpath, 9*fac))) return false;
	return true;
}

void FindFile (const char* path, const char* filename, char* result){
	DIR *hdir;
	struct dirent *entry;
	hdir = opendir(path);
	char newpath[200], nextsearch[200];
	bool done = false;
	sprintf (result, "none");

	do
	{
		entry = readdir(hdir);

		if (entry){
			if (entry->d_type == DT_DIR){
				sprintf (newpath, "%s/%s", path, entry->d_name);
				if ((strcmp(entry->d_name,"."))&&(strcmp(entry->d_name,".."))){
					FindFile (newpath, filename, result);
					if (strcmp(result,"none")) done = true;
				}
			}
			else if (entry->d_type == DT_REG){
				if (!strcmp(entry->d_name, filename)){
					sprintf (result, "%s/%s", path, entry->d_name);
					done = true;
				}
			}
		}

	} while ((entry)&&(!done));
	closedir(hdir);
}

#endif


//EXPERIMENTAL:

//reduce alpha blending
SDL_Surface *killalpha (SDL_Surface *surface, Uint8 alpha){ // remove alpha channel, use color key instead
	SDL_Surface *dummy;
	dummy = SDL_CreateRGBSurface(SDL_SRCCOLORKEY, 1, 1, 32, 0xff000000, 0x00ff0000, 0x0000ff00, 0x00000000);
	surface = SDL_ConvertSurface(surface, dummy->format, SDL_SRCCOLORKEY);
	Uint32 pixel = GetPixel(surface, 0, 0);
	Uint8 R,G,B;
	SDL_GetRGB(pixel, surface->format, &R, &G, &B);
	SDL_SetColorKey(surface, SDL_SRCCOLORKEY, (R,G,B));
	SDL_SetAlpha(surface, SDL_SRCALPHA|SDL_RLEACCEL, alpha);
	return surface;
}


/* // *** Render Text with SDL_Pango ***

SDL_Surface *RenderText (const char *text, Uint8 size, const char *color, bool bold){
	if (SDLPango_WasInit() == 0) SDLPango_Init();
	SDLPango_Context *context = SDLPango_CreateContext();
	SDLPango_SetDefaultColor(context, MATRIX_TRANSPARENT_BACK_WHITE_LETTER);
	char markup[400];
	if (!bold) sprintf (markup, "<span font='%d' color='%s'>%s</span>", size, color, text);
	else sprintf (markup, "<span font='%d' color='%s'><b>%s</b></span>", size, color, text);
	SDLPango_SetMarkup(context, markup, -1);
	Uint32 w = SDLPango_GetLayoutWidth(context);
    Uint32 h = SDLPango_GetLayoutHeight(context);
    Uint8 mx = 0; //margin
    Uint8 my = 0;
    SDL_Surface *surface;
	surface = SDL_CreateRGBSurface(SDL_SWSURFACE, w+mx*2, h+my*2, 32, 0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
	SDLPango_Draw(context, surface, mx, my);
	SDLPango_FreeContext(context);
	return surface;
}
*/


