/*
    Copyright 2008 - 2011 Denis Comtesse <comtesse@users.sourceforge.net>

    This file is part of F.L.A.W.

    F.L.A.W. is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    F.L.A.W. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with F.L.A.W.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "wizard.h"
#ifdef __linux__
//#	define PATH "data/"
#	define PATH PACKAGE_DATA_DIR"/flaw/"
#else
#	define PATH "data/"
#endif



void WizardGFX::load (const char *name, bool ghost, float screenfactor, bool hq) {
	Uint8 alpha;
	if (ghost == true) alpha = 128;
	else alpha = 255;
	char filename[60];
	SDL_Surface *scalecopy;

	if (hq == true) {
		sprintf (filename, PATH"sprites/extras/shield.png", name);
		if (!(scalecopy = IMG_Load (filename))){
			cout << IMG_GetError();
			exit(1);
		}
	}
	else {
		scalecopy = SDL_CreateRGBSurface(0, 120, 120, 32, 0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);		
		for (Uint8 n = 0; n < 5; n++){
			circleRGBA(scalecopy, 60, 60, 50-n, 131, 148, 242, 255);
		}
	}
	
	if (screenfactor == 2) shield = scalecopy;
	else shield = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, hq);
	if (hq == true) SDL_SetAlpha(shield, SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
	else shield=killalpha(shield, 128);


	if (!ghost){
		for (Sint8 d = 0; d < 8; d++){
			for (Sint8 f = 0; f < 8; f++){
				char filename[60];
				sprintf (filename, PATH"sprites/%s/walk_%d_%02d.png", name, d, 2*f+1);
				if (!(scalecopy = IMG_Load (filename))){
					cout << IMG_GetError();
					exit(1);
				}
				if (screenfactor == 2) walk[d][f] = scalecopy;
				else {
					walk[d][f] = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, hq);
					SDL_FreeSurface(scalecopy);
				}
				if (hq == true) SDL_SetAlpha(walk[d][f], SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
				else walk[d][f]=killalpha(walk[d][f], alpha);
			}
		}
	}
	else{
		for (Sint8 d = 0; d < 8; d++){
			char filename[60];
			sprintf (filename, PATH"sprites/%s/walk_%d_01.png", name, d);
			if (!(scalecopy = IMG_Load (filename))){
				cout << IMG_GetError();
				exit(1);
			}
			if (screenfactor == 2) walk[d][0] = scalecopy;
			else {
				walk[d][0] = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, hq);
				SDL_FreeSurface(scalecopy);
			}
			if (hq == true) SDL_SetAlpha(walk[d][0], SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
			else walk[d][0]=killalpha(walk[d][0], alpha);
		}
	}
	
	for (Sint8 d = 0; d < 8; d++){
		for (Sint8 f = 0; f < 10; f++){
			char filename[60];
			int chk;
			
			chk = sprintf (filename, PATH"sprites/%s/hit_%d_%02d.png", name, d, f+1);
			if (!(scalecopy = IMG_Load (filename))){
				cout << IMG_GetError();
			}
			if (screenfactor == 2) hit[d][f] = scalecopy;
			else {
				hit[d][f] = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, hq);
				SDL_FreeSurface(scalecopy);
			}
			if (hq == true) SDL_SetAlpha(hit[d][f], SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
			else hit[d][f]=killalpha(hit[d][f], alpha);
		}
	}

	if (!ghost){
	for (Sint8 d = 0; d < 8; d++){
		for (Sint8 f = 0; f < 8; f++){
			char filename[60];
			int chk;
			
			chk = sprintf (filename, PATH"sprites/%s/fall_%d_%02d.png", name, d, f+1);
			if (!(scalecopy = IMG_Load (filename))){
				cout << IMG_GetError();
			}
			if (screenfactor == 2) fall[d][f] = scalecopy;
			else {
				fall[d][f] = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, hq);
				SDL_FreeSurface(scalecopy);
			}
			if (hq == true) SDL_SetAlpha(fall[d][f], SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
			else fall[d][f]=killalpha(fall[d][f], alpha);
		}
	}
	for (Sint8 d = 0; d < 8; d++){
		for (Sint8 f = 0; f < 8; f++){
			char filename[60];
			int chk;
			
			chk = sprintf (filename, PATH"sprites/%s/fall_4_03.png", name);
			if (!(scalecopy = IMG_Load (filename))){
				cout << IMG_GetError();
			}
			falldown[d][f] = rotozoomSurface(scalecopy, 360-d*45, screenfactor/2*pow(0.75,f), hq);
			SDL_FreeSurface(scalecopy);
			if (hq == true) SDL_SetAlpha(falldown[d][f], SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
			else falldown[d][f]=killalpha(falldown[d][f], alpha);
		}
	}	
	}
	
	for (Sint8 f = 0; f < 6; f++){
		char filename[60];
		int chk;
		
		chk = sprintf (filename, PATH"sprites/burn/burn_%02d.png", f+1);
		if (!(scalecopy = IMG_Load (filename))){
			cout << IMG_GetError();
		}
		if (screenfactor == 2) burn[f] = scalecopy;
		else {
			burn[f] = zoomSurface(scalecopy, screenfactor/2, screenfactor/2, 1);
			SDL_FreeSurface(scalecopy);
		}
		if (hq == true) SDL_SetAlpha(burn[f], SDL_SRCALPHA|SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
		else burn[f]=killalpha(burn[f], 255);
	}
}

void Wizard::setup(Uint8 n, float screenfactor) {
	w = 60*screenfactor;
	h = 60*screenfactor;
	rect.w = w;
	rect.h = h;
	store.w = rect.w+2;
	store.h = rect.h+2;
	frame = 0;
	status = 0;
	direction = 0;
	control.x = 0;
	control.y = 0;
	control.fire = false;
	blitted = false;
	gfxset = n;
	extra = 0;
	timer = 0;
}

void Wizard::updatecontroller(void) {
	
	Uint8 *keystate = SDL_GetKeyState (0);
	
	if (keyboard == 1){  //ARROW KEYS

		if (keystate [SDLK_UP]){
			control.y = -1;
		}
		
		else if (keystate [SDLK_DOWN]){
			control.y = 1;
		}
		
		else {
			control.y = 0;
		}
		
		if (keystate [SDLK_LEFT]){
			control.x = -1;
		}
		
		else if (keystate [SDLK_RIGHT]){
			control.x = 1;
		}
		
		else {
			control.x = 0;
		}
		
		if ((keystate [SDLK_RCTRL]) || (keystate [SDLK_RSHIFT]) || (keystate [SDLK_RETURN])){
			control.fire = true;
		}
		
		else {
			control.fire = false;
		}
	}
	
	else if (keyboard == 2){  //WASD
		if (keystate [SDLK_w]){
			control.y = -1;
		}
		
		else if (keystate [SDLK_s]){
			control.y = 1;
		}
		
		else {
			control.y = 0;
		}
		
		if (keystate [SDLK_a]){
			control.x = -1;
		}
		
		else if (keystate [SDLK_d]){
			control.x = 1;
		}
		
		else {
			control.x = 0;
		}
		
		if ((keystate [SDLK_LCTRL]) || (keystate [SDLK_LSHIFT]) || (keystate [SDLK_SPACE])){
			control.fire = true;
		}
		
		else {
			control.fire = false;
		}
	}
	
	else if (keyboard == 3){  //KEYPAD
		
		if (keystate [SDLK_KP8]){
			control.y = -1;
		}
		
		else if ((keystate[SDLK_KP2]) || (keystate [SDLK_KP5])){
			control.y = 1;
		}
		
		else {
			control.y = 0;
		}
		
		if (keystate [SDLK_KP4]){
			control.x = -1;
		}
		
		else if (keystate [SDLK_KP6]){
			control.x = 1;
		}
		
		else {
			control.x = 0;
		}
		
		/*//diagonal keys:
		if (keystate [SDLK_KP7]){
			control.x = -1;
			control.y = -1;
		}
		else if (keystate [SDLK_KP9]){
			control.x = 1;
			control.y = -1;
		}	
		else if (keystate [SDLK_KP3]){
			control.x = 1;
			control.y = 1;
		}	
		else if (keystate [SDLK_KP1]){
			control.x = -1;
			control.y = 1;
		}	
		*/
		
		if ((keystate [SDLK_KP0]) || (keystate [SDLK_KP_ENTER])){
			control.fire = true;
		}
		
		else {
			control.fire = false;
		}
	}
	
	else if (joystick == true){
		SDL_JoystickUpdate();
		if (SDL_JoystickGetAxis(joy, 0) < -8000){
			control.x = -1;
		}
		
		else if (SDL_JoystickGetAxis(joy, 0) > 8000){
			control.x = 1;
		}
		
		else {
			control.x = 0;
		}
		
		if (SDL_JoystickGetAxis(joy, 1) < -8000){
			control.y = -1;
		}
		
		else if (SDL_JoystickGetAxis(joy, 1) > 8000){
			control.y = 1;
		}
		
		else {
			control.y = 0;
		}
		
		if ((SDL_JoystickGetButton(joy, 0)) || (SDL_JoystickGetButton(joy, 1))){
			control.fire = true;
		}
		
		else {
			control.fire = false;
		}
	}
}


void Wizard::setspeeds(void){
	//setspeeds
	
	if (status < 300) {
		if (extra == 3) {
			if ((control.x != 0) && (control.y != 0)) {
				speed.x = control.x*3.182;
				speed.y = control.y*2.65;
			}
			else {
				speed.x = control.x*4.5;
				speed.y = control.y*3.75;
			}
		}
		else {
			if ((control.x != 0) && (control.y != 0)) {
				speed.x = control.x*2.121;
				speed.y = control.y*1.768;
			}
			else {
				speed.x = control.x*3;
				speed.y = control.y*2.5;
			}
		}
	}
	
	if ((control.fire) && (status < 200)){
		status = 200;
	}
}

void Wizard::wallcollisions(void) {

	//Wizard collides with Wall

	if ((x < 48) && ((control.x < 0)||(prevspeed.x < 0))){
		control.x = 0;
		speed.x = 0;
		prevspeed.x = 0;		
	}
	else if ((x > 600) && ((control.x > 0)||(prevspeed.x > 0))){
		control.x = 0;
		speed.x = 0;
		prevspeed.x = 0;			
	}

	if ((y < 26) && ((control.y < 0)||(prevspeed.y < 0))){
		control.y = 0;		
		speed.y = 0;
		prevspeed.y = 0;
	}
	else if ((y > 430) && ((control.y > 0)||(prevspeed.y > 0))){
		control.y = 0;
		speed.y = 0;
		prevspeed.y = 0; 
	}		
}


//Update Status and calculate the correct frame:

void Wizard::statusupdate(void) {
	
	if (extra > 0) {
		timer--;
		if (timer == 0) {
			extra = 0;
			follows = -1;
			yoffset = 0;
		}
	}

	if (status < 100){
		if ((speed.x!= 0) || (speed.y != 0)) status = 100;
	}
	else if (status < 200){
		if ((speed.x == 0) && (speed.y == 0)) status = 0;
	}
	

	if (status >= 450){    // fall into a hole
		speed.x = 0;
		speed.y = 0;
		prevspeed.x = 0;
		prevspeed.y = 0;
		frame =  (status - 450)/3;
		status++;
		if (status > 473){
			status = 500;
		}
	}
	
	else if (status >= 400){			//BURN
		speed.x = 0;
		speed.y = 0;
		prevspeed.x = 0;
		prevspeed.y = 0;
		frame = (status - 400)/2;
		status++;
		if (status > 411){
			status = 500;
		}
	}
	
	else if (status >= 390){	//stand up
		speed.x = 0;
		speed.y = 0;
		frame = (status - 390)/2 + 4;
		status++;
		if (status > 397){
			status = 0;
		}
	}
	
	else if (status >= 330){	//knocked out

		speed.x = prevspeed.x*0.87;
		speed.y = prevspeed.y*0.87;
		prevspeed.x = speed.x;
		prevspeed.y = speed.y;
		
		frame = 4;
		status++;
	}
	
	else if (status >= 300){	//FALL
		prevspeed.x = 0;
		prevspeed.y = 0;
		speed.x = 0;
		speed.y = 0;
		frame = (status - 300)/2;
		status++;
		if (status > 307){
			status = 330;
			if ((direction == UP)&&(skid==true)){
				prevspeed.y = 2;
			}
			if ((direction == UPRIGHT)&&(skid==true)){
				prevspeed.y = 1.414;
				prevspeed.x = -1.414;
			}
			if ((direction == RIGHT)&&(skid==true)){
				prevspeed.x = -2;
			}
			if ((direction == DOWNRIGHT)&&(skid==true)){
				prevspeed.y = -1.414;
				prevspeed.x = -1.414;
			}
			if ((direction == DOWN)&&(skid==true)){
				prevspeed.y = -2;
			}
			if ((direction == DOWNLEFT)&&(skid==true)){
				prevspeed.y = -1.414;
				prevspeed.x = 1.414;
			}
			if ((direction == LEFT)&&(skid==true)){
				prevspeed.x = 2;
			}
			if ((direction == UPLEFT)&&(skid==true)){
				prevspeed.y = 1.414;
				prevspeed.x = 1.414;
			}
		}
	}
	
	else if (status >= 200){	//HIT
		speed.x = 0;
		speed.y = 0;
		if ((skid==true)&&(extra != 4)){
			speed.x = prevspeed.x*0.92 + speed.x*0.08;
			speed.y = prevspeed.y*0.92 + speed.y*0.08;
			prevspeed.x = speed.x;
			prevspeed.y = speed.y;
		}
		
		frame = 1+(status - 200)/2;
		status++;
		if (status > 217){
			status = 0;
		}
	}
	
	else if (status >= 100){	//WALK
		if ((skid==true)&&(extra != 4)){
			speed.x = prevspeed.x*0.92 + speed.x*0.08;
			speed.y = prevspeed.y*0.92 + speed.y*0.08;
			prevspeed.x = speed.x;
			prevspeed.y = speed.y;
		}
		
		frame = (status - 100)/3;
		status++;
		if (status > 123){
			status = 100;
		}
		
		//Directions:
		if (control.x > 0){
			
			if (control.y > 0){
				direction = DOWNRIGHT;
			}
			
			else if (control.y < 0){
				direction = UPRIGHT;
			}
			
			else {
				direction = RIGHT;
			}
		}
		
		else if (control.x < 0){
			
			if (control.y > 0){
				direction = DOWNLEFT;
			}
			
			else if (control.y < 0){
				direction = UPLEFT;
			}
			
			else {
				direction = LEFT;
			}
		}
		
		else{	
			
			if (control.y > 0){
				direction = DOWN;
			}

			else if (control.y < 0){
				direction = UP;

			}			
		}
	}
	
	else if (status < 100){			//STAND
		speed.x = 0;
		speed.y = 0;
		if ((skid==true)&&(extra != 4)){
			speed.x = prevspeed.x*0.92 + speed.x*0.08;
			speed.y = prevspeed.y*0.92 + speed.y*0.08;
			prevspeed.x = speed.x;
			prevspeed.y = speed.y;
		}
		frame = 0;
		status = 0;
	}
	
	x+=speed.x;
	y+=speed.y;
	
}


//Restore Background:

void Wizard::blitrestore (SDL_Surface *backup, float screenoffset) {
	SDL_Surface *dst = SDL_GetVideoSurface();
	if (blitted == true) {
		SDL_Rect srcrect = rect;
		srcrect.x -= screenoffset;
		SDL_BlitSurface (backup, &srcrect, dst, &rect);
		blitted = false;
	}
}


//blit:

void Wizard::blit (WizardGFX *set, float screenfactor, float screenoffset) {
	
	rect.x = screenfactor*x-w/2+screenoffset;
	rect.y = screenfactor*y-h/2+yoffset;
	SDL_Surface *dst = SDL_GetVideoSurface();

	
	if (status < 500) {
		if (status >= 450) {
			rect.x = screenfactor*x-set->falldown[direction][frame]->w/2+screenoffset;
			rect.y = screenfactor*y-set->falldown[direction][frame]->h/2;
			SDL_BlitSurface (set->falldown[direction][frame], NULL, dst, &rect);
			blitted = true;
		}
		else if (status >= 400) {
			SDL_BlitSurface (set->burn[frame], NULL, dst, &rect);
			blitted = true;
		}
		
		else if (status >= 300){
			SDL_BlitSurface (set->fall[direction][frame], NULL, dst, &rect);
			blitted = true;
		}
		
		else if (status >= 200){
			SDL_BlitSurface (set->hit[direction][frame], NULL, dst, &rect);
			blitted = true;
		}
		
		else if (status >= 100){
			if (extra==4) frame=0;
			SDL_BlitSurface (set->walk[direction][frame], NULL, dst, &rect);
			blitted = true;
		}

		else if (status < 100){
			SDL_BlitSurface (set->walk[direction][0], NULL, dst, &rect);
			blitted = true;
		}
		
		if ((extra == 2) && (status < 400)) {
			if ((timer > 50)||(timer%2 == 0)) {
				SDL_BlitSurface (set->shield, NULL, dst, &rect);
				blitted = true;
			}
		}
	}
}

